@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Implementation of Cabin
- *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2006 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -14,6 +14,8 @@
  *************************************************************************************************/
 
 
+#define QDBM_INTERNAL  1
+
 #include "cabin.h"
 #include "myconf.h"
 
@@ -1485,7 +1487,7 @@ CBLIST *cbdirlist(const char *name){
 int cbfilestat(const char *name, int *isdirp, int *sizep, int *mtimep){
   struct stat sbuf;
   assert(name);
-  if(stat(name, &sbuf) == -1) return FALSE;
+  if(lstat(name, &sbuf) == -1) return FALSE;
   if(isdirp) *isdirp = S_ISDIR(sbuf.st_mode);
   if(sizep) *sizep = (int)sbuf.st_size;
   if(mtimep) *mtimep = (int)sbuf.st_mtime;
@@ -2370,6 +2372,38 @@ unsigned int cbgetcrc(const char *ptr, int size){
 }
 
 
+/* Compress a serial object with LZO. */
+char *cblzoencode(const char *ptr, int size, int *sp){
+  assert(ptr && sp);
+  if(!_qdbm_lzoencode) return NULL;
+  return _qdbm_lzoencode(ptr, size, sp);
+}
+
+
+/* Decompress a serial object compressed with LZO. */
+char *cblzodecode(const char *ptr, int size, int *sp){
+  assert(ptr && size >= 0);
+  if(!_qdbm_lzodecode) return NULL;
+  return _qdbm_lzodecode(ptr, size, sp);
+}
+
+
+/* Compress a serial object with BZIP2. */
+char *cbbzencode(const char *ptr, int size, int *sp){
+  assert(ptr && sp);
+  if(!_qdbm_bzencode) return NULL;
+  return _qdbm_bzencode(ptr, size, sp);
+}
+
+
+/* Decompress a serial object compressed with BZIP2. */
+char *cbbzdecode(const char *ptr, int size, int *sp){
+  assert(ptr && size >= 0);
+  if(!_qdbm_bzdecode) return NULL;
+  return _qdbm_bzdecode(ptr, size, sp);
+}
+
+
 /* Convert the character encoding of a string. */
 char *cbiconv(const char *ptr, int size, const char *icode, const char *ocode, int *sp, int *mp){
   assert(ptr && icode && ocode);
@@ -2697,6 +2731,32 @@ void *cbmyfatal(const char *message){
 }
 
 
+/* Create a datum handle from an allocated region. */
+CBDATUM *cbdatumopenbuf(char *ptr, int size){
+  CBDATUM *datum;
+  assert(ptr && size >= 0);
+  CB_REALLOC(ptr, size + 1);
+  CB_MALLOC(datum, sizeof(*datum));
+  datum->dptr = ptr;
+  datum->dptr[size] = '\0';
+  datum->dsize = size;
+  datum->asize = size;
+  return datum;
+}
+
+
+/* Set a buffer to a datum handle. */
+void cbdatumsetbuf(CBDATUM *datum, char *ptr, int size){
+  assert(datum && ptr && size >= 0);
+  free(datum->dptr);
+  CB_REALLOC(ptr, size + 1);
+  datum->dptr = ptr;
+  datum->dptr[size] = '\0';
+  datum->dsize = size;
+  datum->asize = size;
+}
+
+
 /* Add an allocated element at the end of a list. */
 void cblistpushbuf(CBLIST *list, char *ptr, int size){
   int index;
