@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Popular encoders and decoders
- *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2006 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -50,6 +50,8 @@ int runmime(int argc, char **argv);
 int runcsv(int argc, char **argv);
 int runxml(int argc, char **argv);
 int runzlib(int argc, char **argv);
+int runlzo(int argc, char **argv);
+int runbzip(int argc, char **argv);
 int runiconv(int argc, char **argv);
 int rundate(int argc, char **argv);
 void shouucsmap(void);
@@ -76,6 +78,10 @@ int main(int argc, char **argv){
     rv = runxml(argc, argv);
   } else if(!strcmp(argv[1], "zlib")){
     rv = runzlib(argc, argv);
+  } else if(!strcmp(argv[1], "lzo")){
+    rv = runlzo(argc, argv);
+  } else if(!strcmp(argv[1], "bzip")){
+    rv = runbzip(argc, argv);
   } else if(!strcmp(argv[1], "iconv")){
     rv = runiconv(argc, argv);
   } else if(!strcmp(argv[1], "date")){
@@ -105,6 +111,14 @@ void usage(void){
     fprintf(stderr, "  %s zlib [-d] [-gz] [-crc] [file]\n", progname);
     free(tmp);
   }
+  if((tmp = cblzoencode("", 0, &tsiz)) != NULL){
+    fprintf(stderr, "  %s lzo [-d] [file]\n", progname);
+    free(tmp);
+  }
+  if((tmp = cbbzencode("", 0, &tsiz)) != NULL){
+    fprintf(stderr, "  %s bzip [-d] [file]\n", progname);
+    free(tmp);
+  }
   if((tmp = cbiconv("", 0, "US-ASCII", "US-ASCII", NULL, NULL)) != NULL){
     fprintf(stderr, "  %s iconv [-ic code] [-oc code] [-ol ltype] [-cn] [-um] [file]\n",
             progname);
@@ -740,6 +754,114 @@ int runzlib(int argc, char **argv){
 }
 
 
+/* parse arguments of lzo command */
+int runlzo(int argc, char **argv){
+  int i, bsiz, rsiz, dec;
+  char *file, *buf, *res;
+  dec = FALSE;
+  file = NULL;
+  for(i = 2; i < argc; i++){
+    if(!file && argv[i][0] == '-'){
+      if(!strcmp(argv[i], "-d")){
+        dec = TRUE;
+      } else {
+        usage();
+      }
+    } else if(!file){
+      file = argv[i];
+    } else {
+      usage();
+    }
+  }
+  buf = NULL;
+  if(file){
+    if(!(buf = cbreadfile(file, &bsiz))){
+      fprintf(stderr, "%s: %s: cannot open\n", progname, file);
+      return 1;
+    }
+  } else {
+    buf = readstdin(&bsiz);
+  }
+  if(dec){
+    if(!(res = cblzodecode(buf, bsiz, &rsiz))){
+      fprintf(stderr, "%s: decode failed\n", progname);
+      free(buf);
+      return 1;
+    }
+    for(i = 0; i < rsiz; i++){
+      putchar(res[i]);
+    }
+    free(res);
+  } else {
+    if(!(res = cblzoencode(buf, bsiz, &rsiz))){
+      fprintf(stderr, "%s: encode failed\n", progname);
+      free(buf);
+      return 1;
+    }
+    for(i = 0; i < rsiz; i++){
+      putchar(res[i]);
+    }
+    free(res);
+  }
+  free(buf);
+  return 0;
+}
+
+
+/* parse arguments of bzip command */
+int runbzip(int argc, char **argv){
+  int i, bsiz, rsiz, dec;
+  char *file, *buf, *res;
+  dec = FALSE;
+  file = NULL;
+  for(i = 2; i < argc; i++){
+    if(!file && argv[i][0] == '-'){
+      if(!strcmp(argv[i], "-d")){
+        dec = TRUE;
+      } else {
+        usage();
+      }
+    } else if(!file){
+      file = argv[i];
+    } else {
+      usage();
+    }
+  }
+  buf = NULL;
+  if(file){
+    if(!(buf = cbreadfile(file, &bsiz))){
+      fprintf(stderr, "%s: %s: cannot open\n", progname, file);
+      return 1;
+    }
+  } else {
+    buf = readstdin(&bsiz);
+  }
+  if(dec){
+    if(!(res = cbbzdecode(buf, bsiz, &rsiz))){
+      fprintf(stderr, "%s: decode failed\n", progname);
+      free(buf);
+      return 1;
+    }
+    for(i = 0; i < rsiz; i++){
+      putchar(res[i]);
+    }
+    free(res);
+  } else {
+    if(!(res = cbbzencode(buf, bsiz, &rsiz))){
+      fprintf(stderr, "%s: encode failed\n", progname);
+      free(buf);
+      return 1;
+    }
+    for(i = 0; i < rsiz; i++){
+      putchar(res[i]);
+    }
+    free(res);
+  }
+  free(buf);
+  return 0;
+}
+
+
 /* parse arguments of iconv command */
 int runiconv(int argc, char **argv){
   CBDATUM *datum;
