@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Test cases of Curia
- *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2006 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -55,7 +55,8 @@ void pdperror(const char *name);
 int myrand(void);
 int dowrite(const char *name, int rnum, int bnum, int dnum, int sparse, int lob);
 int doread(const char *name, int wb, int lob);
-int dorcat(const char *name, int rnum, int bnum, int dnum, int pnum, int align, int cb);
+int dorcat(const char *name, int rnum, int bnum, int dnum, int pnum,
+           int align, int fbpsiz, int cb);
 int docombo(const char *name);
 int dowicked(const char *name, int rnum, int cb);
 
@@ -93,7 +94,7 @@ void usage(void){
   fprintf(stderr, "usage:\n");
   fprintf(stderr, "  %s write [-s] [-lob] name rnum bnum dnum\n", progname);
   fprintf(stderr, "  %s read [-wb] [-lob] name\n", progname);
-  fprintf(stderr, "  %s rcat [-c] name rnum bnum dnum pnum align\n", progname);
+  fprintf(stderr, "  %s rcat [-c] name rnum bnum dnum pnum align fbpsiz\n", progname);
   fprintf(stderr, "  %s combo name\n", progname);
   fprintf(stderr, "  %s wicked [-c] name rnum\n", progname);
   fprintf(stderr, "\n");
@@ -175,18 +176,20 @@ int runread(int argc, char **argv){
 
 /* parse arguments of rcat command */
 int runrcat(int argc, char **argv){
-  char *name, *rstr, *bstr, *dstr, *pstr, *astr;
-  int i, rnum, bnum, dnum, pnum, align, cb, rv;
+  char *name, *rstr, *bstr, *dstr, *pstr, *astr, *fstr;
+  int i, rnum, bnum, dnum, pnum, align, fbpsiz, cb, rv;
   name = NULL;
   rstr = NULL;
   bstr = NULL;
   dstr = NULL;
   pstr = NULL;
   astr = NULL;
+  fstr = NULL;
   rnum = 0;
   bnum = 0;
   pnum = 0;
   align = 0;
+  fbpsiz = 0;
   cb = FALSE;
   for(i = 2; i < argc; i++){
     if(!name && argv[i][0] == '-'){
@@ -207,18 +210,21 @@ int runrcat(int argc, char **argv){
       pstr = argv[i];
     } else if(!astr){
       astr = argv[i];
+    } else if(!fstr){
+      fstr = argv[i];
     } else {
       usage();
     }
   }
-  if(!name || !rstr || !bstr || !dstr || !pstr || !astr) usage();
+  if(!name || !rstr || !bstr || !dstr || !pstr || !astr || !fstr) usage();
   rnum = atoi(rstr);
   bnum = atoi(bstr);
   dnum = atoi(dstr);
   pnum = atoi(pstr);
   align = atoi(astr);
-  if(rnum < 1 || bnum < 1 || dnum < 1 || pnum < 1) usage();
-  rv = dorcat(name, rnum, bnum, dnum, pnum, align, cb);
+  fbpsiz = atoi(fstr);
+  if(rnum < 1 || bnum < 1 || dnum < 1 || pnum < 1 || fbpsiz < 0) usage();
+  rv = dorcat(name, rnum, bnum, dnum, pnum, align, fbpsiz, cb);
   return rv;
 }
 
@@ -413,19 +419,20 @@ int doread(const char *name, int wb, int lob){
 
 
 /* perform rcat command */
-int dorcat(const char *name, int rnum, int bnum, int dnum, int pnum, int align, int cb){
+int dorcat(const char *name, int rnum, int bnum, int dnum, int pnum,
+           int align, int fbpsiz, int cb){
   CURIA *curia;
   CBMAP *map;
   int i, err, len, ksiz, vsiz, rsiz;
   const char *kbuf, *vbuf;
   char buf[RECBUFSIZ], *rbuf;
   printfflush("<Random Writing Test>\n  name=%s  rnum=%d  bnum=%d  dnum=%d  pnum=%d"
-              "  align=%d  c=%d\n\n", name, rnum, bnum, dnum, pnum, align, cb);
+              "  align=%d  fbpsiz=%d  c=%d\n\n", name, rnum, bnum, dnum, pnum, align, fbpsiz, cb);
   if(!(curia = cropen(name, CR_OWRITER | CR_OCREAT | CR_OTRUNC, bnum, dnum))){
     pdperror(name);
     return 1;
   }
-  if(!crsetalign(curia, align)){
+  if(!crsetalign(curia, align) || !crsetfbpsiz(curia, fbpsiz)){
     pdperror(name);
     crclose(curia);
     return 1;
@@ -703,7 +710,7 @@ int dowicked(const char *name, int rnum, int cb){
     pdperror(name);
     return 1;
   }
-  if(!crsetalign(curia, 16)){
+  if(!crsetalign(curia, 16) || !crsetfbpsiz(curia, 256)){
     pdperror(name);
     err = TRUE;
   }
