@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Implementation of Depot
- *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2006 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -14,6 +14,8 @@
  *************************************************************************************************/
 
 
+#define QDBM_INTERNAL  1
+
 #include "depot.h"
 #include "myconf.h"
 
@@ -21,17 +23,16 @@
 #define DP_MAGICNUMB   "[DEPOT]\n\f"     /* magic number on environments of big endian */
 #define DP_MAGICNUML   "[depot]\n\f"     /* magic number on environments of little endian */
 #define DP_HEADSIZ     48                /* size of the reagion of the header */
-#define DP_LIBVEROFF   12                /* offset of the region for flags */
+#define DP_LIBVEROFF   12                /* offset of the region for the library version */
 #define DP_FLAGSOFF    16                /* offset of the region for flags */
 #define DP_FSIZOFF     24                /* offset of the region for the file size */
 #define DP_BNUMOFF     32                /* offset of the region for the bucket number */
 #define DP_RNUMOFF     40                /* offset of the region for the record number */
 #define DP_DEFBNUM     8191              /* default bucket number */
 #define DP_FBPOOLSIZ   16                /* size of free block pool */
-#define DP_COALMAX     3                 /* number of trials for record coalescence */
 #define DP_ENTBUFSIZ   128               /* size of the entity buffer */
 #define DP_STKBUFSIZ   256               /* size of the stack key buffer */
-#define DP_WRTBUFSIZ   2048              /* size of the writing buffer */
+#define DP_WRTBUFSIZ   8192              /* size of the writing buffer */
 #define DP_TMPFSUF     MYEXTSTR "dptmp"  /* suffix of a temporary file */
 #define DP_OPTBLOAD    0.25              /* ratio of bucket loading at optimization */
 #define DP_OPTRUNIT    256               /* number of records in a process of optimization */
@@ -128,8 +129,9 @@ const char *dperrmsg(int ecode){
 /* Get a database handle. */
 DEPOT *dpopen(const char *name, int omode, int bnum){
   char hbuf[DP_HEADSIZ], *map, c, *tname;
-  int i, mode, fd, inode, mtime, fsiz, rnum, msiz, *fbpool;
+  int i, mode, fd, inode, fsiz, rnum, msiz, *fbpool;
   struct stat sbuf;
+  time_t mtime;
   DEPOT *depot;
   assert(name);
   mode = O_RDONLY;
@@ -210,7 +212,7 @@ DEPOT *dpopen(const char *name, int omode, int bnum){
   }
   if((dpbigendian() ? memcmp(hbuf, DP_MAGICNUMB, strlen(DP_MAGICNUMB)) != 0 :
       memcmp(hbuf, DP_MAGICNUML, strlen(DP_MAGICNUML)) != 0) ||
-     memcmp(hbuf + DP_FSIZOFF, &fsiz, sizeof(int)) != 0){
+     *((int *)(hbuf + DP_FSIZOFF)) != fsiz){
     close(fd);
     dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
     return NULL;
@@ -232,7 +234,7 @@ DEPOT *dpopen(const char *name, int omode, int bnum){
   tname = NULL;
   fbpool = NULL;
   if(!(depot = malloc(sizeof(DEPOT))) || !(tname = dpstrdup(name)) ||
-     !(fbpool = malloc(DP_FBPOOLSIZ * sizeof(int) * 2))){
+     !(fbpool = malloc(DP_FBPOOLSIZ * 2 * sizeof(int)))){
     free(fbpool);
     free(tname);
     free(depot);
@@ -259,6 +261,7 @@ DEPOT *dpopen(const char *name, int omode, int bnum){
     depot->fbpool[i] = -1;
     depot->fbpool[i+1] = -1;
   }
+  depot->fbpsiz = DP_FBPOOLSIZ * 2;
   depot->align = 0;
   return depot;
 }
@@ -298,7 +301,7 @@ int dpclose(DEPOT *depot){
 /* Store a record. */
 int dpput(DEPOT *depot, const char *kbuf, int ksiz, const char *vbuf, int vsiz, int dmode){
   int head[DP_RHNUM], next[DP_RHNUM];
-  int i, j, hash, bi, off, entoff, ee, newoff, rsiz, nsiz, fdel, mroff, mrsiz, mi, min;
+  int i, hash, bi, off, entoff, ee, newoff, rsiz, nsiz, fdel, mroff, mrsiz, mi, min;
   char ebuf[DP_ENTBUFSIZ], *tval, *swap;
   assert(depot && kbuf && vbuf);
   if(depot->fatal){
@@ -337,19 +340,18 @@ int dpput(DEPOT *depot, const char *kbuf, int ksiz, const char *vbuf, int vsiz,
         depot->fsiz = off + rsiz;
       }
     } else {
-      for(i = 0; i < DP_COALMAX && nsiz > rsiz && off + rsiz < depot->fsiz; i++){
-        for(j = 0; j < DP_FBPOOLSIZ * 2; j += 2){
-          if(depot->fbpool[j] == off + rsiz){
-            depot->fbpool[j] = -1;
-            depot->fbpool[j+1] = -1;
-            break;
-          }
-        }
+      while(nsiz > rsiz && off + rsiz < depot->fsiz){
         if(!dprechead(depot, off + rsiz, next, NULL, NULL)) return FALSE;
         if(!(next[DP_RHIFLAGS] & DP_RECFREUSE)) break;
         head[DP_RHIPSIZ] += dprecsize(next);
         rsiz += dprecsize(next);
       }
+      for(i = 0; i < depot->fbpsiz; i += 2){
+        if(depot->fbpool[i] >= off && depot->fbpool[i] < off + rsiz){
+          depot->fbpool[i] = -1;
+          depot->fbpool[i+1] = -1;
+        }
+      }
     }
     if(nsiz <= rsiz){
       if(!dprecover(depot, off, head, vbuf, vsiz, dmode == DP_DCAT)){
@@ -385,7 +387,7 @@ int dpput(DEPOT *depot, const char *kbuf, int ksiz, const char *vbuf, int vsiz,
       }
       mi = -1;
       min = -1;
-      for(i = 0; i < DP_FBPOOLSIZ * 2; i += 2){
+      for(i = 0; i < depot->fbpsiz; i += 2){
         if(depot->fbpool[i+1] < nsiz) continue;
         if(mi == -1 || depot->fbpool[i+1] < min){
           mi = i;
@@ -673,6 +675,35 @@ int dpsetalign(DEPOT *depot, int align){
 }
 
 
+/* Set the size of the free block pool of a database handle. */
+int dpsetfbpsiz(DEPOT *depot, int size){
+  int *fbpool;
+  int i;
+  assert(depot && size >= 0);
+  if(depot->fatal){
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
+    return FALSE;
+  }
+  if(!depot->wmode){
+    dpecodeset(DP_EMODE, __FILE__, __LINE__);
+    return FALSE;
+  }
+  size *= 2;
+  if(!(fbpool = realloc(depot->fbpool, size * sizeof(int) + 1))){
+    dpecodeset(DP_EALLOC, __FILE__, __LINE__);
+    return FALSE;
+  }
+  for(i = 0; i < size; i += 2){
+    fbpool[i] = -1;
+    fbpool[i+1] = -1;
+  }
+  depot->fbpool = fbpool;
+  depot->fbpsiz = size;
+  return TRUE;
+}
+
+
+
 /* Synchronize contents of updating a database with the file and the device. */
 int dpsync(DEPOT *depot){
   assert(depot);
@@ -825,7 +856,7 @@ int dpoptimize(DEPOT *depot, int bnum){
   depot->fsiz = tdepot->fsiz;
   depot->bnum = tdepot->bnum;
   depot->ioff = 0;
-  for(i = 0; i < DP_FBPOOLSIZ * 2; i += 2){
+  for(i = 0; i < depot->fbpsiz; i += 2){
     depot->fbpool[i] = -1;
     depot->fbpool[i+1] = -1;
   }
@@ -948,7 +979,7 @@ int dpinode(DEPOT *depot){
 
 
 /* Get the last modified time of a database. */
-int dpmtime(DEPOT *depot){
+time_t dpmtime(DEPOT *depot){
   assert(depot);
   return depot->mtime;
 }
@@ -1224,7 +1255,7 @@ int dpprimenum(int num){
 
 
 /* Name of the operating system. */
-char *dpsysname = _QDBM_SYSNAME;
+const char *dpsysname = _QDBM_SYSNAME;
 
 
 /* File descriptor for debugging output. */
@@ -1289,7 +1320,10 @@ int dpmemsync(DEPOT *depot){
 /* Get flags of a database. */
 int dpgetflags(DEPOT *depot){
   assert(depot);
-  return *(depot->map + DP_FLAGSOFF);
+  return (*(unsigned char *)(depot->map + DP_FLAGSOFF + 0) << 0) +
+    (*(unsigned char *)(depot->map + DP_FLAGSOFF + 1) << 8) +
+    (*(unsigned char *)(depot->map + DP_FLAGSOFF + 2) << 16) +
+    (*(unsigned char *)(depot->map + DP_FLAGSOFF + 3) << 24);
 }
 
 
@@ -1300,7 +1334,10 @@ int dpsetflags(DEPOT *depot, int flags){
     dpecodeset(DP_EMODE, __FILE__, __LINE__);
     return FALSE;
   }
-  *(depot->map + DP_FLAGSOFF) = flags;
+  *(depot->map + DP_FLAGSOFF + 0) = (flags >> 0) & 0xff;
+  *(depot->map + DP_FLAGSOFF + 1) = (flags >> 8) & 0xff;
+  *(depot->map + DP_FLAGSOFF + 2) = (flags >> 16) & 0xff;
+  *(depot->map + DP_FLAGSOFF + 3) = (flags >> 24) & 0xff;
   return TRUE;
 }
 
@@ -1907,7 +1944,7 @@ static int dprecappend(DEPOT *depot, const char *kbuf, int ksiz, const char *vbu
 static int dprecover(DEPOT *depot, int off, int *head, const char *vbuf, int vsiz, int cat){
   int i, hsiz, hoff, voff;
   assert(depot && off >= 0 && head && vbuf && vsiz >= 0);
-  for(i = 0; i < DP_FBPOOLSIZ * 2; i += 2){
+  for(i = 0; i < depot->fbpsiz; i += 2){
     if(depot->fbpool[i] == off){
       depot->fbpool[i] = -1;
       depot->fbpool[i+1] = -1;
@@ -1942,26 +1979,26 @@ static int dprecover(DEPOT *depot, int off, int *head, const char *vbuf, int vsi
    `reusable' specifies whether the region is reusable or not.
    The return value is true if successful, or, false on failure. */
 static int dprecdelete(DEPOT *depot, int off, int *head, int reusable){
-  int i, mi, min;
+  int i, mi, min, size;
   assert(depot && off >= 0 && head);
   if(reusable){
     mi = -1;
     min = -1;
-    for(i = 0; i < DP_FBPOOLSIZ * 2; i += 2){
+    for(i = 0; i < depot->fbpsiz; i += 2){
       if(depot->fbpool[i] == -1){
         depot->fbpool[i] = off;
         depot->fbpool[i+1] = dprecsize(head);
         mi = -1;
         break;
       }
-      if(min < 0 || depot->fbpool[i+1] < min){
+      if(mi == -1 || depot->fbpool[i+1] < min){
         mi = i;
         min = depot->fbpool[i+1];
       }
     }
-    if(mi >= 0){
+    if(mi >= 0 && (size = dprecsize(head)) > min){
       depot->fbpool[mi] = off;
-      depot->fbpool[mi+1] = dprecsize(head);
+      depot->fbpool[mi+1] = size;
     }
   }
   return dpseekwritenum(depot->fd, off + DP_RHIFLAGS * sizeof(int),
