@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * The basic API of QDBM
- *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2006 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -23,6 +23,14 @@ extern "C" {
 
 
 #include <stdlib.h>
+#include <time.h>
+
+
+#if defined(_MSC_VER) && !defined(QDBM_INTERNAL) && !defined(QDBM_STATIC)
+#define MYEXTERN extern __declspec(dllimport)
+#else
+#define MYEXTERN extern
+#endif
 
 
 
@@ -33,9 +41,9 @@ extern "C" {
 
 typedef struct {                         /* type of structure for a database handle */
   char *name;                            /* name of the database file */
-  int wmode;                             /* whether writable or not */
+  int wmode;                             /* whether to be writable */
   int inode;                             /* inode of the database file */
-  int mtime;                             /* last modified time of the database */
+  time_t mtime;                          /* last modified time of the database */
   int fd;                                /* file descriptor of the database file */
   int fsiz;                              /* size of the database file */
   char *map;                             /* pointer to the mapped memory */
@@ -43,9 +51,10 @@ typedef struct {                         /* type of structure for a database han
   int *buckets;                          /* pointer to the bucket array */
   int bnum;                              /* number of the bucket array */
   int rnum;                              /* number of records */
-  int fatal;                             /* whether a fatal error occured or not */
+  int fatal;                             /* whether a fatal error occured */
   int ioff;                              /* offset of the iterator */
   int *fbpool;                           /* free block pool */
+  int fbpsiz;                            /* size of the free block pool */
   int align;                             /* basic size of alignment */
 } DEPOT;
 
@@ -91,7 +100,7 @@ enum {                                   /* enumeration for write modes */
 
 
 /* String containing the version information. */
-extern const char *dpversion;
+MYEXTERN const char *dpversion;
 
 
 /* Last happened error code. */
@@ -235,7 +244,7 @@ char *dpiternext(DEPOT *depot, int *sp);
    `depot' specifies a database handle connected as a writer.
    `align' specifies the size of alignment.
    If successful, the return value is true, else, it is false.
-   If alignment is set to a database, the efficiency of overwriting values are improved.
+   If alignment is set to a database, the efficiency of overwriting values is improved.
    The size of alignment is suggested to be average size of the values of the records to be
    stored.  If alignment is positive, padding whose size is multiple number of the alignment
    is placed.  If alignment is negative, as `vsiz' is the size of a value, the size of padding
@@ -244,6 +253,15 @@ char *dpiternext(DEPOT *depot, int *sp);
 int dpsetalign(DEPOT *depot, int align);
 
 
+/* Set the size of the free block pool of a database handle.
+   `depot' specifies a database handle connected as a writer.
+   `size' specifies the size of the free block pool of a database.
+   If successful, the return value is true, else, it is false.
+   The default size of the free block pool is 16.  If the size is greater, the space efficiency
+   of overwriting values is improved with the time efficiency sacrificed. */
+int dpsetfbpsiz(DEPOT *depot, int size);
+
+
 /* Synchronize updating contents with the file and the device.
    `depot' specifies a database handle connected as a writer.
    If successful, the return value is true, else, it is false.
@@ -319,7 +337,7 @@ int dpinode(DEPOT *depot);
 /* Get the last modified time of a database.
    `depot' specifies a database handle.
    The return value is the last modified time of the database. */
-int dpmtime(DEPOT *depot);
+time_t dpmtime(DEPOT *depot);
 
 
 /* Get the file descriptor of a database file.
@@ -390,20 +408,20 @@ int dpprimenum(int num);
  *************************************************************************************************/
 
 
-#define _QDBM_VERSION  "1.8.36"
-#define _QDBM_LIBVER   1108
+#define _QDBM_VERSION  "1.8.44"
+#define _QDBM_LIBVER   1205
 
 
 /* Name of the operating system. */
-extern char *dpsysname;
+MYEXTERN const char *dpsysname;
 
 
 /* File descriptor for debugging output. */
-extern int dpdbgfd;
+MYEXTERN int dpdbgfd;
 
 
 /* Whether this build is reentrant. */
-extern const int dpisreentrant;
+MYEXTERN const int dpisreentrant;
 
 
 /* Set the last happened error code.
@@ -431,12 +449,14 @@ int dpgetflags(DEPOT *depot);
 
 /* Set flags of a database.
    `depot' specifies a database handle connected as a writer.
-   `flags' specifies flags to set.
+   `flags' specifies flags to set.  Least ten bits are reserved for internal use.
    If successful, the return value is true, else, it is false. */
 int dpsetflags(DEPOT *depot, int flags);
 
 
 
+#undef MYEXTERN
+
 #if defined(__cplusplus)                 /* export for C++ */
 }
 #endif
