@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Test cases of Depot
- *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2006 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -54,7 +54,7 @@ void pdperror(const char *name);
 int myrand(void);
 int dowrite(const char *name, int rnum, int bnum, int sparse);
 int doread(const char *name, int wb);
-int dorcat(const char *name, int rnum, int bnum, int pnum, int align, int cb);
+int dorcat(const char *name, int rnum, int bnum, int pnum, int align, int fbpsiz, int cb);
 int docombo(const char *name);
 int dowicked(const char *name, int rnum, int cb);
 
@@ -92,7 +92,7 @@ void usage(void){
   fprintf(stderr, "usage:\n");
   fprintf(stderr, "  %s write [-s] name rnum bnum\n", progname);
   fprintf(stderr, "  %s read [-wb] name\n", progname);
-  fprintf(stderr, "  %s rcat [-c] name rnum bnum pnum align\n", progname);
+  fprintf(stderr, "  %s rcat [-c] name rnum bnum pnum align fbpsiz\n", progname);
   fprintf(stderr, "  %s combo name\n", progname);
   fprintf(stderr, "  %s wicked [-c] name rnum\n", progname);
   fprintf(stderr, "\n");
@@ -163,17 +163,19 @@ int runread(int argc, char **argv){
 
 /* parse arguments of rcat command */
 int runrcat(int argc, char **argv){
-  char *name, *rstr, *bstr, *pstr, *astr;
-  int i, rnum, bnum, pnum, align, cb, rv;
+  char *name, *rstr, *bstr, *pstr, *astr, *fstr;
+  int i, rnum, bnum, pnum, align, fbpsiz, cb, rv;
   name = NULL;
   rstr = NULL;
   bstr = NULL;
   pstr = NULL;
   astr = NULL;
+  fstr = NULL;
   rnum = 0;
   bnum = 0;
   pnum = 0;
   align = 0;
+  fbpsiz = 0;
   cb = FALSE;
   for(i = 2; i < argc; i++){
     if(!name && argv[i][0] == '-'){
@@ -192,17 +194,20 @@ int runrcat(int argc, char **argv){
       pstr = argv[i];
     } else if(!astr){
       astr = argv[i];
+    } else if(!fstr){
+      fstr = argv[i];
     } else {
       usage();
     }
   }
-  if(!name || !rstr || !bstr || !pstr || !astr) usage();
+  if(!name || !rstr || !bstr || !pstr || !astr || !fstr) usage();
   rnum = atoi(rstr);
   bnum = atoi(bstr);
   pnum = atoi(pstr);
   align = atoi(astr);
-  if(rnum < 1 || bnum < 1 || pnum < 1) usage();
-  rv = dorcat(name, rnum, bnum, pnum, align, cb);
+  fbpsiz= atoi(fstr);
+  if(rnum < 1 || bnum < 1 || pnum < 1 || fbpsiz < 0) usage();
+  rv = dorcat(name, rnum, bnum, pnum, align, fbpsiz, cb);
   return rv;
 }
 
@@ -377,19 +382,19 @@ int doread(const char *name, int wb){
 
 
 /* perform rcat command */
-int dorcat(const char *name, int rnum, int bnum, int pnum, int align, int cb){
+int dorcat(const char *name, int rnum, int bnum, int pnum, int align, int fbpsiz, int cb){
   DEPOT *depot;
   CBMAP *map;
   int i, err, len, ksiz, vsiz, rsiz;
   const char *kbuf, *vbuf;
   char buf[RECBUFSIZ], *rbuf;
-  printfflush("<Random Writing Test>\n  name=%s  rnum=%d  bnum=%d  pnum=%d  align=%d  c=%d\n\n",
-              name, rnum, bnum, pnum, align, cb);
+  printfflush("<Random Writing Test>\n  name=%s  rnum=%d  bnum=%d  pnum=%d  align=%d"
+              "  fbpsiz=%d  c=%d\n\n", name, rnum, bnum, pnum, align, fbpsiz, cb);
   if(!(depot = dpopen(name, DP_OWRITER | DP_OCREAT | DP_OTRUNC, bnum))){
     pdperror(name);
     return 1;
   }
-  if(!dpsetalign(depot, align)){
+  if(!dpsetalign(depot, align) || !dpsetfbpsiz(depot, fbpsiz)){
     pdperror(name);
     dpclose(depot);
     return 1;
@@ -668,7 +673,7 @@ int dowicked(const char *name, int rnum, int cb){
     pdperror(name);
     return 1;
   }
-  if(!dpsetalign(depot, 16)){
+  if(!dpsetalign(depot, 16) || !dpsetfbpsiz(depot, 256)){
     pdperror(name);
     err = TRUE;
   }
