@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Java API of Depot, the basic API of QDBM
- *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2006 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -313,7 +313,7 @@ public class Depot implements ADBM {
    * Set alignment of the database handle.
    * @param align the basic size of alignment.
    * @throws DepotException if an error occurs.
-   * @note If alignment is set to a database, the efficiency of overwriting values are improved.
+   * @note If alignment is set to a database, the efficiency of overwriting values is improved.
    * The size of alignment is suggested to be average size of the values of the records to be
    * stored.  If alignment is positive, padding whose size is multiple number of the alignment
    * is placed.  If alignment is negative, as `vsiz' is the size of a value, the size of padding
@@ -326,6 +326,19 @@ public class Depot implements ADBM {
       if(dpsetalign(index, align) == 0) throw new DepotException(dpecode());
     }
   }
+  /**
+   * Set the size of the free block pool.
+   * @param size the size of the free block pool of a database.
+   * @throws DepotException if an error occurs.
+   * @note The default size of the free block pool is 16.  If the size is greater, the space
+   * efficiency of overwriting values is improved with the time efficiency sacrificed.
+   */
+  public void setfbpsiz(int size) throws DepotException {
+    if(index < 0) throw new DepotException();
+    synchronized(ADBM.class){
+      if(dpsetfbpsiz(index, size) == 0) throw new DepotException(dpecode());
+    }
+  }
   /**
    * Synchronize updating contents with the file and the device.
    * @throws DepotException if an error occurs.
@@ -439,6 +452,28 @@ public class Depot implements ADBM {
       return dpfatalerror(index) == 0 ? false : true;
     }
   }
+  /**
+   * Get the inode number of the database.
+   * @return the inode number of the database file.
+   * @throws DepotException if an error occurs.
+   */
+  public int inode() throws DepotException {
+    if(index < 0) throw new DepotException();
+    synchronized(ADBM.class){
+      return dpinode(index);
+    }
+  }
+  /**
+   * Get the last modified time of the database.
+   * @return the last modified time of the database.
+   * @throws DepotException if an error occurs.
+   */
+  public long mtime() throws DepotException {
+    if(index < 0) throw new DepotException();
+    synchronized(ADBM.class){
+      return dpmtime(index);
+    }
+  }
   /**
    * Store a record.
    * @param key a byte array of a key.
@@ -509,6 +544,7 @@ public class Depot implements ADBM {
   private static synchronized final native int dpiterinit(int index);
   private static synchronized final native byte[] dpiternext(int index);
   private static synchronized final native int dpsetalign(int index, int align);
+  private static synchronized final native int dpsetfbpsiz(int index, int size);
   private static synchronized final native int dpsync(int index);
   private static synchronized final native int dpoptimize(int index, int bnum);
   private static synchronized final native String dpname(int index);
@@ -518,6 +554,8 @@ public class Depot implements ADBM {
   private static synchronized final native int dprnum(int index);
   private static synchronized final native int dpwritable(int index);
   private static synchronized final native int dpfatalerror(int index);
+  private static synchronized final native int dpinode(int index);
+  private static synchronized final native long dpmtime(int index);
   private static synchronized final native int dpremove(String name);
 }
 
