@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Implementation of Villa for Java
- *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2006 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -36,6 +36,7 @@ static int getvlomode(jint omode);
 static VLCFUNC getvlcmp(jint cmode);
 static int getvldmode(jint dmode);
 static int getvljmode(jint jmode);
+static int getvlcpmode(jint cpmode);
 static int objcompare(const char *aptr, int asiz, const char *bptr, int bsiz);
 
 
@@ -313,6 +314,29 @@ Java_qdbm_Villa_vlcurval(JNIEnv *env, jclass myclass, jint index)
 }
 
 
+JNIEXPORT jint JNICALL
+Java_qdbm_Villa_vlcurput(JNIEnv *env, jclass myclass, jint index,
+                         jbyteArray val, jint vsiz, jint cpmode){
+  jbyte *vbuf;
+  jboolean icv;
+  int rv;
+  vljnienv = env;
+  vlmyclass = myclass;
+  vbuf = (*env)->GetByteArrayElements(env, val, &icv);
+  rv = vlcurput(vltable[index], vbuf, vsiz, getvlcpmode(cpmode));
+  if(icv == JNI_TRUE) (*env)->ReleaseByteArrayElements(env, val, vbuf, JNI_ABORT);
+  return rv;
+}
+
+
+JNIEXPORT jint JNICALL
+Java_qdbm_Villa_vlcurout(JNIEnv *env, jclass myclass, jint index){
+  vljnienv = env;
+  vlmyclass = myclass;
+  return vlcurout(vltable[index]);
+}
+
+
 JNIEXPORT void JNICALL
 Java_qdbm_Villa_vlsettuning(JNIEnv *env, jclass myclass, jint index,
                             jint lrecmax, jint nidxmax, jint lcnum, jint ncnum)
@@ -413,6 +437,24 @@ Java_qdbm_Villa_vlfatalerror(JNIEnv *env, jclass myclass, jint index)
 }
 
 
+JNIEXPORT jint JNICALL
+Java_qdbm_Villa_vlinode(JNIEnv *env, jclass myclass, jint index)
+{
+  vljnienv = env;
+  vlmyclass = myclass;
+  return vlinode(vltable[index]);
+}
+
+
+JNIEXPORT jlong JNICALL
+Java_qdbm_Villa_vlmtime(JNIEnv *env, jclass myclass, jint index)
+{
+  vljnienv = env;
+  vlmyclass = myclass;
+  return vlmtime(vltable[index]);
+}
+
+
 JNIEXPORT jint JNICALL
 Java_qdbm_Villa_vltranbegin(JNIEnv *env, jclass myclass, jint index)
 {
@@ -490,6 +532,8 @@ static int getvlomode(jint omode){
   if(omode & qdbm_Villa_ONOLCK) vlomode |= VL_ONOLCK;
   if(omode & qdbm_Villa_OLCKNB) vlomode |= VL_OLCKNB;
   if(omode & qdbm_Villa_OZCOMP) vlomode |= VL_OZCOMP;
+  if(omode & qdbm_Villa_OYCOMP) vlomode |= VL_OYCOMP;
+  if(omode & qdbm_Villa_OXCOMP) vlomode |= VL_OXCOMP;
   return vlomode;
 }
 
@@ -509,7 +553,9 @@ static int getvldmode(jint dmode){
   switch(dmode){
   case qdbm_Villa_DOVER: return VL_DOVER;
   case qdbm_Villa_DKEEP: return VL_DKEEP;
+  case qdbm_Villa_DCAT: return VL_DCAT;
   case qdbm_Villa_DDUP: return VL_DDUP;
+  case qdbm_Villa_DDUPR: return VL_DDUPR;
   }
   return -1;
 }
@@ -524,6 +570,16 @@ static int getvljmode(jint jmode){
 }
 
 
+static int getvlcpmode(jint cpmode){
+  switch(cpmode){
+  case qdbm_Villa_CPCURRENT: return VL_CPCURRENT;
+  case qdbm_Villa_CPBEFORE: return VL_CPBEFORE;
+  case qdbm_Villa_CPAFTER: return VL_CPAFTER;
+  }
+  return -1;
+}
+
+
 static int objcompare(const char *aptr, int asiz, const char *bptr, int bsiz){
   jmethodID mid;
   jbyteArray abuf, bbuf;
