@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Emulation of system calls
- *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2006 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -134,8 +134,97 @@ static void *_qdbm_gettsd(void *ptr, int size, const void *initval){
  *************************************************************************************************/
 
 
-#if defined(_SYS_MSVC_) || defined(_SYS_MINGW_) || \
-  defined(_SYS_FREEBSD_) || defined(_SYS_NETBSD_) || defined(_SYS_OPENBSD_) || \
+#if defined(_SYS_MSVC_) || defined(_SYS_MINGW_)
+
+
+#define MMFDESCMAX     2048
+
+
+struct { void *start; HANDLE handle; } mmhandles[MMFDESCMAX];
+int mmhnum = 0;
+CRITICAL_SECTION mmcsec;
+
+
+static void _qdbm_delete_mmap_env(void);
+
+
+void *_qdbm_mmap(void *start, size_t length, int prot, int flags, int fd, off_t offset){
+  static int first = FALSE;
+  HANDLE handle;
+  int i;
+  if(!InterlockedCompareExchange(&first, TRUE, FALSE)){
+    InitializeCriticalSection(&mmcsec);
+    atexit(_qdbm_delete_mmap_env);
+  }
+  if(fd < 0 || flags & MAP_FIXED) return MAP_FAILED;
+  if(!(handle = CreateFileMapping((HANDLE)_get_osfhandle(fd), NULL,
+                                  (prot & PROT_WRITE) ? PAGE_READWRITE : PAGE_READONLY,
+                                  0, length, NULL))) return MAP_FAILED;
+  if(!(start = MapViewOfFile(handle, (prot & PROT_WRITE) ? FILE_MAP_WRITE : FILE_MAP_READ,
+                             0, 0, length))){
+    CloseHandle(handle);
+    return MAP_FAILED;
+  }
+  EnterCriticalSection(&mmcsec);
+  if(mmhnum >= MMFDESCMAX - 1){
+    UnmapViewOfFile(start);
+    CloseHandle(handle);
+    LeaveCriticalSection(&mmcsec);
+    return MAP_FAILED;
+  }
+  for(i = 0; i < MMFDESCMAX; i++){
+    if(!mmhandles[i].start){
+      mmhandles[i].start = start;
+      mmhandles[i].handle = handle;
+      break;
+    }
+  }
+  mmhnum++;
+  LeaveCriticalSection(&mmcsec);
+  return start;
+}
+
+
+int _qdbm_munmap(void *start, size_t length){
+  HANDLE handle;
+  int i;
+  EnterCriticalSection(&mmcsec);
+  handle = NULL;
+  for(i = 0; i < MMFDESCMAX; i++){
+    if(mmhandles[i].start == start){
+      handle = mmhandles[i].handle;
+      mmhandles[i].start = NULL;
+      mmhandles[i].handle = NULL;
+      break;
+    }
+  }
+  if(!handle){
+    LeaveCriticalSection(&mmcsec);
+    return -1;
+  }
+  mmhnum--;
+  LeaveCriticalSection(&mmcsec);
+  if(!UnmapViewOfFile(start)){
+    CloseHandle(handle);
+    return -1;
+  }
+  if(!CloseHandle(handle)) return -1;
+  return 0;
+}
+
+
+int _qdbm_msync(const void *start, size_t length, int flags){
+  if(!FlushViewOfFile(start, length)) return -1;
+  return 0;
+}
+
+
+static void _qdbm_delete_mmap_env(void){
+  DeleteCriticalSection(&mmcsec);
+}
+
+
+#elif defined(_SYS_FREEBSD_) || defined(_SYS_NETBSD_) || defined(_SYS_OPENBSD_) || \
   defined(_SYS_RISCOS_) || defined(MYNOMMAP)
 
 
@@ -327,11 +416,17 @@ int _qdbm_win32_fcntl(int fd, int cmd, struct flock *lock){
 
 
 DIR *_qdbm_win32_opendir(const char *name){
-  char expr[1024];
+  char expr[8192];
+  int len;
   DIR *dir;
   HANDLE fh;
   WIN32_FIND_DATA data;
-  sprintf(expr, "%s%c*", name, MYPATHCHR);
+  len = strlen(name);
+  if(len > 0 && name[len-1] == MYPATHCHR){
+    sprintf(expr, "%s*", name);
+  } else {
+    sprintf(expr, "%s%c*", name, MYPATHCHR);
+  }
   if((fh = FindFirstFile(expr, &data)) == INVALID_HANDLE_VALUE) return NULL;
   if(!(dir = malloc(sizeof(DIR)))){
     FindClose(fh);
@@ -416,7 +511,8 @@ static char *_qdbm_deflate_impl(const char *ptr, int size, int *sp, int mode){
       return NULL;
     break;
   }
-  asiz = ZLIBBUFSIZ;
+  asiz = size + 16;
+  if(asiz < ZLIBBUFSIZ) asiz = ZLIBBUFSIZ;
   if(!(buf = malloc(asiz))){
     deflateEnd(&zs);
     return NULL;
@@ -459,7 +555,8 @@ static char *_qdbm_deflate_impl(const char *ptr, int size, int *sp, int mode){
   }
   memcpy(buf + bsiz, obuf, osiz);
   bsiz += osiz;
-  if(mode == _QDBM_ZMRAW) buf[bsiz++] = '\0';
+  buf[bsiz] = '\0';
+  if(mode == _QDBM_ZMRAW) bsiz++;
   *sp = bsiz;
   deflateEnd(&zs);
   return buf;
@@ -485,9 +582,10 @@ static char *_qdbm_inflate_impl(const char *ptr, int size, int *sp, int mode){
     if(inflateInit2(&zs, 15) != Z_OK) return NULL;
     break;
   }
-  asiz = ZLIBBUFSIZ;
+  asiz = size * 2 + 16;
+  if(asiz < ZLIBBUFSIZ) asiz = ZLIBBUFSIZ;
   if(!(buf = malloc(asiz))){
-    deflateEnd(&zs);
+    inflateEnd(&zs);
     return NULL;
   }
   bsiz = 0;
@@ -501,7 +599,7 @@ static char *_qdbm_inflate_impl(const char *ptr, int size, int *sp, int mode){
       asiz = asiz * 2 + osiz;
       if(!(swap = realloc(buf, asiz))){
         free(buf);
-        deflateEnd(&zs);
+        inflateEnd(&zs);
         return NULL;
       }
       buf = swap;
@@ -521,7 +619,7 @@ static char *_qdbm_inflate_impl(const char *ptr, int size, int *sp, int mode){
     asiz = asiz * 2 + osiz;
     if(!(swap = realloc(buf, asiz))){
       free(buf);
-      deflateEnd(&zs);
+      inflateEnd(&zs);
       return NULL;
     }
     buf = swap;
@@ -555,6 +653,237 @@ unsigned int (*_qdbm_getcrc)(const char *, int) = NULL;
 
 
 
+/*************************************************************************************************
+ * for LZO
+ *************************************************************************************************/
+
+
+#if defined(MYLZO)
+
+
+#include <lzo/lzo1x.h>
+
+
+static char *_qdbm_lzoencode_impl(const char *ptr, int size, int *sp);
+static char *_qdbm_lzodecode_impl(const char *ptr, int size, int *sp);
+
+
+int _qdbm_lzo_init = FALSE;
+char *(*_qdbm_lzoencode)(const char *, int, int *) = _qdbm_lzoencode_impl;
+char *(*_qdbm_lzodecode)(const char *, int, int *) = _qdbm_lzodecode_impl;
+
+
+static char *_qdbm_lzoencode_impl(const char *ptr, int size, int *sp){
+  char wrkmem[LZO1X_1_MEM_COMPRESS];
+  lzo_bytep buf;
+  lzo_uint bsiz;
+  if(!_qdbm_lzo_init){
+    if(lzo_init() != LZO_E_OK) return NULL;
+    _qdbm_lzo_init = TRUE;
+  }
+  if(size < 0) size = strlen(ptr);
+  if(!(buf = malloc(size + size / 16 + 80))) return NULL;
+  if(lzo1x_1_compress((lzo_bytep)ptr, size, buf, &bsiz, wrkmem) != LZO_E_OK){
+    free(buf);
+    return NULL;
+  }
+  buf[bsiz] = '\0';
+  *sp = bsiz;
+  return (char *)buf;
+}
+
+
+static char *_qdbm_lzodecode_impl(const char *ptr, int size, int *sp){
+  lzo_bytep buf;
+  lzo_uint bsiz;
+  int rat, rv;
+  if(!_qdbm_lzo_init){
+    if(lzo_init() != LZO_E_OK) return NULL;
+    _qdbm_lzo_init = TRUE;
+  }
+  rat = 6;
+  while(TRUE){
+    bsiz = (size + 256) * rat + 3;
+    if(!(buf = malloc(bsiz + 1))) return NULL;
+    rv = lzo1x_decompress_safe((lzo_bytep)(ptr), size, buf, &bsiz, NULL);
+    if(rv == LZO_E_OK){
+      break;
+    } else if(rv == LZO_E_OUTPUT_OVERRUN){
+      free(buf);
+      rat *= 2;
+    } else {
+      free(buf);
+      return NULL;
+    }
+  }
+  buf[bsiz] = '\0';
+  if(sp) *sp = bsiz;
+  return (char *)buf;
+}
+
+
+#else
+
+
+char *(*_qdbm_lzoencode)(const char *, int, int *) = NULL;
+char *(*_qdbm_lzodecode)(const char *, int, int *) = NULL;
+
+
+#endif
+
+
+
+/*************************************************************************************************
+ * for BZIP2
+ *************************************************************************************************/
+
+
+#if defined(MYBZIP)
+
+
+#include <bzlib.h>
+
+#define BZIPBUFSIZ     8192
+
+
+static char *_qdbm_bzencode_impl(const char *ptr, int size, int *sp);
+static char *_qdbm_bzdecode_impl(const char *ptr, int size, int *sp);
+
+
+char *(*_qdbm_bzencode)(const char *, int, int *) = _qdbm_bzencode_impl;
+char *(*_qdbm_bzdecode)(const char *, int, int *) = _qdbm_bzdecode_impl;
+
+
+static char *_qdbm_bzencode_impl(const char *ptr, int size, int *sp){
+  bz_stream zs;
+  char *buf, *swap, obuf[BZIPBUFSIZ];
+  int rv, asiz, bsiz, osiz;
+  if(size < 0) size = strlen(ptr);
+  zs.bzalloc = NULL;
+  zs.bzfree = NULL;
+  zs.opaque = NULL;
+  if(BZ2_bzCompressInit(&zs, 9, 0, 30) != BZ_OK) return NULL;
+  asiz = size + 16;
+  if(asiz < BZIPBUFSIZ) asiz = BZIPBUFSIZ;
+  if(!(buf = malloc(asiz))){
+    BZ2_bzCompressEnd(&zs);
+    return NULL;
+  }
+  bsiz = 0;
+  zs.next_in = (char *)ptr;
+  zs.avail_in = size;
+  zs.next_out = obuf;
+  zs.avail_out = BZIPBUFSIZ;
+  while((rv = BZ2_bzCompress(&zs, BZ_FINISH)) == BZ_FINISH_OK){
+    osiz = BZIPBUFSIZ - zs.avail_out;
+    if(bsiz + osiz > asiz){
+      asiz = asiz * 2 + osiz;
+      if(!(swap = realloc(buf, asiz))){
+        free(buf);
+        BZ2_bzCompressEnd(&zs);
+        return NULL;
+      }
+      buf = swap;
+    }
+    memcpy(buf + bsiz, obuf, osiz);
+    bsiz += osiz;
+    zs.next_out = obuf;
+    zs.avail_out = BZIPBUFSIZ;
+  }
+  if(rv != BZ_STREAM_END){
+    free(buf);
+    BZ2_bzCompressEnd(&zs);
+    return NULL;
+  }
+  osiz = BZIPBUFSIZ - zs.avail_out;
+  if(bsiz + osiz + 1 > asiz){
+    asiz = asiz * 2 + osiz;
+    if(!(swap = realloc(buf, asiz))){
+      free(buf);
+      BZ2_bzCompressEnd(&zs);
+      return NULL;
+    }
+    buf = swap;
+  }
+  memcpy(buf + bsiz, obuf, osiz);
+  bsiz += osiz;
+  buf[bsiz] = '\0';
+  *sp = bsiz;
+  BZ2_bzCompressEnd(&zs);
+  return buf;
+}
+
+
+static char *_qdbm_bzdecode_impl(const char *ptr, int size, int *sp){
+  bz_stream zs;
+  char *buf, *swap, obuf[BZIPBUFSIZ];
+  int rv, asiz, bsiz, osiz;
+  zs.bzalloc = NULL;
+  zs.bzfree = NULL;
+  zs.opaque = NULL;
+  if(BZ2_bzDecompressInit(&zs, 0, 0) != BZ_OK) return NULL;
+  asiz = size * 2 + 16;
+  if(asiz < BZIPBUFSIZ) asiz = BZIPBUFSIZ;
+  if(!(buf = malloc(asiz))){
+    BZ2_bzDecompressEnd(&zs);
+    return NULL;
+  }
+  bsiz = 0;
+  zs.next_in = (char *)ptr;
+  zs.avail_in = size;
+  zs.next_out = obuf;
+  zs.avail_out = BZIPBUFSIZ;
+  while((rv = BZ2_bzDecompress(&zs)) == BZ_OK){
+    osiz = BZIPBUFSIZ - zs.avail_out;
+    if(bsiz + osiz >= asiz){
+      asiz = asiz * 2 + osiz;
+      if(!(swap = realloc(buf, asiz))){
+        free(buf);
+        BZ2_bzDecompressEnd(&zs);
+        return NULL;
+      }
+      buf = swap;
+    }
+    memcpy(buf + bsiz, obuf, osiz);
+    bsiz += osiz;
+    zs.next_out = obuf;
+    zs.avail_out = BZIPBUFSIZ;
+  }
+  if(rv != BZ_STREAM_END){
+    free(buf);
+    BZ2_bzDecompressEnd(&zs);
+    return NULL;
+  }
+  osiz = BZIPBUFSIZ - zs.avail_out;
+  if(bsiz + osiz >= asiz){
+    asiz = asiz * 2 + osiz;
+    if(!(swap = realloc(buf, asiz))){
+      free(buf);
+      BZ2_bzDecompressEnd(&zs);
+      return NULL;
+    }
+    buf = swap;
+  }
+  memcpy(buf + bsiz, obuf, osiz);
+  bsiz += osiz;
+  buf[bsiz] = '\0';
+  if(sp) *sp = bsiz;
+  BZ2_bzDecompressEnd(&zs);
+  return buf;
+}
+
+
+#else
+
+
+char *(*_qdbm_bzencode)(const char *, int, int *) = NULL;
+char *(*_qdbm_bzdecode)(const char *, int, int *) = NULL;
+
+
+#endif
+
+
+
 /*************************************************************************************************
  * for ICONV
  *************************************************************************************************/
