@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Utility for debugging Odeum and its applications
- *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2006 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -117,7 +117,7 @@ int main(int argc, char **argv){
     rv = runbreak(argc, argv);
   } else if(!strcmp(argv[1], "version") || !strcmp(argv[1], "--version")){
     printf("Powered by QDBM version %s\n", dpversion);
-    printf("Copyright (c) 2000-2005 Mikio Hirabayashi\n");
+    printf("Copyright (c) 2000-2006 Mikio Hirabayashi\n");
     rv = 0;
   } else {
     usage();
@@ -979,7 +979,7 @@ int doinform(const char *name){
   printf("all documents: %d\n", oddnum(odeum));
   printf("all words: %d\n", odwnum(odeum));
   printf("inode number: %d\n", odinode(odeum));
-  printf("modified time: %d\n", odmtime(odeum));
+  printf("modified time: %.0f\n", (double)odmtime(odeum));
   if(!odclose(odeum)){
     pdperror(name);
     return 1;
