@@ -301,7 +301,7 @@ sub iternext {
 # Method: Set alignment of the database handle.
 # `$align' specifies the basic size of alignment.  If it is undef, alignment is cleared.
 # If successful, the return value is true, else, it is false.
-# If alignment is set to a database, the efficiency of overwriting values are improved.
+# If alignment is set to a database, the efficiency of overwriting values is improved.
 # The size of alignment is suggested to be average size of the values of the records to be
 # stored.  If alignment is positive, padding whose size is multiple number of the alignment
 # is placed.  If alignment is negative, as `vsiz' is the size of a value, the size of padding
@@ -321,6 +321,28 @@ sub setalign {
 }
 
 
+##
+# $bool = $depot->setfbpsiz($size);
+# Method: Set the size of the free block pool.
+# `$size' specifies the size of the free block pool.  If it is undef, the free block pool is not
+# used.
+# If successful, the return value is true, else, it is false.
+# The default size of the free block pool is 16.  If the size is greater, the space efficiency of
+# overwriting values is improved with the time efficiency sacrificed.
+#
+sub setfbpsiz {
+    my($self) = shift;
+    ($$self[1]) || return FALSE;
+    my($size) = shift;
+    (defined($size)) || ($size = 0);
+    (scalar(@_) == 0) || return FALSE;
+    my($depot) = $handles{$$self[0]};
+    my($rv) = pldpsetfbpsiz($depot, $size);
+    $errmsg = pldperrmsg();
+    return $rv;
+}
+
+
 ##
 # $bool = $depot->sync();
 # Method: Synchronize updating contents with the file and the device.
