@@ -35,7 +35,9 @@ use constant OCREAT => 1 << 2;           # a writer creating
 use constant OTRUNC => 1 << 3;           # a writer truncating
 use constant ONOLCK => 1 << 4;           # open without locking
 use constant OLCKNB => 1 << 5;           # lock without blocking
-use constant OZCOMP => 1 << 6;           # compress leaves
+use constant OZCOMP => 1 << 6;           # compress leaves with ZLIB
+use constant OYCOMP => 1 << 7;           # compress leaves with LZO
+use constant OXCOMP => 1 << 8;           # compress leaves with BZIP2
 
 use constant CMPLEX => 0;                # compare in lexical order
 use constant CMPDEC => 1;                # compare decimal strings
@@ -44,10 +46,15 @@ use constant DOVER => 0;                 # overwrite an existing value
 use constant DKEEP => 1;                 # keep an existing value
 use constant DCAT => 2;                  # concatenate values
 use constant DDUP => 3;                  # allow duplication of records
+use constant DDUPR => 4;                 # allow duplication with reverse order
 
 use constant JFORWARD => 0;              # step forward
 use constant JBACKWARD => 1;             # step backward
 
+use constant CPCURRENT => 0;             # overwrite the current record
+use constant CPBEFORE => 1;              # insert before the current record
+use constant CPAFTER => 2;               # insert after the current record
+
 my(%handles) = ();                       # table of database names
 our($errmsg) = "no error";               # message of the last error
 
@@ -79,18 +86,21 @@ our($errmsg) = "no error";               # message of the last error
 # reader.  If the mode is `Villa::OWRITER', the following may be added by bitwise or:
 # `Villa::OCREAT', which means it creates a new database if not exist, `Villa::OTRUNC', which
 # means it creates a new database regardless if one exists, `Villa::OZCOMP', which means leaves
-# in the database are compressed.  Both of `Villa::OREADER' and `Villa::OWRITER' can be added to
-# by bitwise or: `Villa::ONOLCK', which means it opens a database file without file locking, or
-# `Villa::OLCKNB', which means locking is performed without blocking.  If it is undef,
-# `Villa::OREADER' is specified.
-# `cmode' specifies the comparing function: `Villa::CMPLEX' comparing keys in lexical order,
+# in the database are compressed with ZLIB, `Villa::OYCOMP', which means leaves in the database
+# are compressed with LZO, `Villa::OXCOMP', which means leaves in the database are compressed
+# with BZIP2.  Both of `Villa::OREADER' and `Villa::OWRITER' can be added to by bitwise or:
+# `Villa::ONOLCK', which means it opens a database file without file locking, or `Villa::OLCKNB',
+# which means locking is performed without blocking.  If it is undef, `Villa::OREADER' is
+# specified.
+# `$cmode' specifies the comparing function: `Villa::CMPLEX' comparing keys in lexical order,
 # `Villa::CMPDEC' comparing keys as decimal strings.  The comparing function should be kept
 # same in the life of a database.
 # The return value is the database handle or undef if it is not successful.
 # While connecting as a writer, an exclusive lock is invoked to the database file.
 # While connecting as a reader, a shared lock is invoked to the database file.  The thread
-# blocks until the lock is achieved.  `Villa::OZCOMP' is available only if QDBM was built with
-# ZLIB enabled.  If `Villa::ONOLCK' is used, the application is responsible for exclusion control.
+# blocks until the lock is achieved.  `Villa::OZCOMP', `Villa::OYCOMP', and `Villa::OXCOMP' are
+# available only if QDBM was built each with ZLIB, LZO, and BZIP2 enabled.  If `Villa::ONOLCK'
+# is used, the application is responsible for exclusion control.
 #
 sub new {
     my($class) = shift;
@@ -141,11 +151,12 @@ sub close {
 # `$dmode' specifies behavior when the key overlaps, by the following values: `Villa::DOVER',
 # which means the specified value overwrites the existing one, `Villa::DKEEP', which means the
 # existing value is kept, `Villa::DCAT', which means the specified value is concatenated at the
-# end of the existing value, `Villa::DDUP', which means duplication of keys is allowed.  If it is
-# undef, `Villa::DOVER' is specified.
+# end of the existing value, `Villa::DDUP', which means duplication of keys is allowed and the
+# specified value is added as the last one, `Villa::DDUPR', which means duplication of keys is
+# allowed and the specified value is added as the first one.  If it is undef, `Villa::DOVER' is
+# specified.
 # If successful, the return value is true, else, it is false.
-# A duplicated record is stored at the tail of the records of the same key.  The cursor becomes
-# unavailable due to updating database.
+# The cursor becomes unavailable due to updating database.
 #
 sub put {
     my($self) = shift;
@@ -428,6 +439,56 @@ sub curval {
 }
 
 
+##
+# $bool = $villa->curput($val, $cpmode);
+# Method: Insert a record around the cursor.
+# `$val' specifies a value.  If it is undef, this method has no effect.
+# `$cpmode' specifies detail adjustment: `Villa::CPCURRENT', which means that the value of the
+# current record is overwritten, `Villa::CPBEFORE', which means that a new record is inserted
+# before the current record, `Villa::CPAFTER', which means that a new record is inserted after
+# the current record.  If it is undef, `Villa::CPCURRENT' is specified.
+# If successful, the return value is true, else, it is false.  False is returned when no record
+# corresponds to the cursor.
+# After insertion, the cursor is moved to the inserted record.
+#
+sub curput {
+    my($self) = shift;
+    ($$self[1]) || return FALSE;
+    my($val) = shift;
+    my($cpmode) = shift;
+    (scalar(@_) == 0) || return FALSE;
+    (defined($val)) || return FALSE;
+    (defined($cpmode)) || ($cpmode = CPCURRENT);
+    my($villa) = $handles{$$self[0]};
+    if($$self[3]){
+        local($_) = $val;
+        $$self[3]();
+        $val = $_;
+    }
+    my($rv) = plvlcurput($villa, $val, length($val), $cpmode);
+    $errmsg = plvlerrmsg();
+    return $rv;
+}
+
+
+##
+# $bool = $villa->curout();
+# Method: Delete the record where the cursor is.
+# If successful, the return value is true, else, it is false.  False is returned when no record
+# corresponds to the cursor.
+# After deletion, the cursor is moved to the next record if possible.
+#
+sub curout {
+    my($self) = shift;
+    ($$self[1]) || return FALSE;
+    (scalar(@_) == 0) || return FALSE;
+    my($villa) = $handles{$$self[0]};
+    my($rv) = plvlcurout($villa);
+    $errmsg = plvlerrmsg();
+    return $rv;
+}
+
+
 ##
 # $bool = $villa->settuning($lrecmax, $nidxmax, $lcnum, $ncnum);
 # Method: Set alignment of the database handle.
@@ -480,7 +541,7 @@ sub sync {
 
 
 ##
-# $bool = $villa->optimize($bnum);
+# $bool = $villa->optimize();
 # Method: Optimize the database.
 # If successful, the return value is true, else, it is false.
 # In an alternating succession of deleting and storing with overwrite or concatenate,
