@@ -17,6 +17,7 @@
 #include "xcuria.h"
 #include <new>
 #include <cstdlib>
+#include <ctime>
 
 extern "C" {
 #include <depot.h>
@@ -290,6 +291,17 @@ void Curia::setalign(int align) throw(Curia_error){
 }
 
 
+void Curia::setfbpsiz(int size) throw(Curia_error){
+  if(!curia) throw Curia_error();
+  if(pthread_mutex_lock(&mutex) != 0) throw Curia_error();
+  if(!crsetfbpsiz(curia, size)){
+    pthread_mutex_unlock(&mutex);
+    throw Curia_error(dpecode);
+  }
+  pthread_mutex_unlock(&mutex);
+}
+
+
 void Curia::sync() throw(Curia_error){
   if(!curia) throw Curia_error();
   if(pthread_mutex_lock(&mutex) != 0) throw Curia_error();
@@ -421,8 +433,8 @@ int Curia::inode() throw(Curia_error){
 }
 
 
-int Curia::mtime() throw(Curia_error){
-  int rv;
+time_t Curia::mtime() throw(Curia_error){
+  time_t rv;
   if(!curia) throw Curia_error();
   if(pthread_mutex_lock(&mutex) != 0) throw Curia_error();
   rv = crmtime(curia);
