@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Implementation of Villa
- *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2006 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -14,6 +14,8 @@
  *************************************************************************************************/
 
 
+#define QDBM_INTERNAL  1
+
 #include "villa.h"
 #include "myconf.h"
 
@@ -21,6 +23,7 @@
 #define VL_NODEIDMIN   100000000         /* minimum number of node ID */
 #define VL_VNUMBUFSIZ  8                 /* size of a buffer for variable length number */
 #define VL_NUMBUFSIZ   32                /* size of a buffer for a number */
+#define VL_PAGEBUFSIZ  32768             /* size of a buffer to read each page */
 #define VL_LEVELMAX    64                /* max level of B+ tree */
 #define VL_DEFLRECMAX  49                /* default number of records in each leaf */
 #define VL_DEFNIDXMAX  192               /* default number of indexes in each node */
@@ -31,6 +34,7 @@
 #define VL_INITBNUM    32749             /* initial bucket number */
 #define VL_INITALIGN   448               /* initial size of alignment */
 #define VL_OPTALIGN    -3                /* alignment setting when optimization */
+#define VL_FBPOOLSIZ   64                /* size of free block pool */
 #define VL_PATHBUFSIZ  1024              /* size of a path buffer */
 #define VL_TMPFSUF     MYEXTSTR "vltmp"  /* suffix of a temporary file */
 #define VL_ROOTKEY     -1                /* key of the root key */
@@ -40,8 +44,10 @@
 #define VL_RNUMKEY     -5                /* key of the number of records */
 
 enum {                                   /* enumeration for flags */
-  VL_FLISVILLA = 1 << 0,                 /* whether for Villa or not */
-  VL_FLISZLIB = 1 << 1                   /* whether with ZLIB or not */
+  VL_FLISVILLA = 1 << 0,                 /* whether for Villa */
+  VL_FLISZLIB = 1 << 1,                  /* whether with ZLIB */
+  VL_FLISLZO = 1 << 2,                   /* whether with LZO */
+  VL_FLISBZIP = 1 << 3                   /* whether with BZIP2 */
 };
 
 
@@ -88,7 +94,7 @@ VLCFUNC VL_CMPDEC = vldeccompare;
 /* Get a database handle. */
 VILLA *vlopen(const char *name, int omode, VLCFUNC cmp){
   DEPOT *depot;
-  int dpomode, flags, zmode, root, last, lnum, nnum, rnum;
+  int dpomode, flags, cmode, root, last, lnum, nnum, rnum;
   VILLA *villa;
   VLLEAF *leaf;
   assert(name && cmp);
@@ -102,7 +108,7 @@ VILLA *vlopen(const char *name, int omode, VLCFUNC cmp){
   if(omode & VL_OLCKNB) dpomode |= DP_OLCKNB;
   if(!(depot = dpopen(name, dpomode, VL_INITBNUM))) return NULL;
   flags = dpgetflags(depot);
-  zmode = 0;
+  cmode = 0;
   root = -1;
   last = -1;
   lnum = 0;
@@ -118,14 +124,32 @@ VILLA *vlopen(const char *name, int omode, VLCFUNC cmp){
       dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
       return NULL;
     }
-    zmode = flags & VL_FLISZLIB;
+    if(flags & VL_FLISZLIB){
+      cmode = VL_OZCOMP;
+    } else if(flags & VL_FLISLZO){
+      cmode = VL_OYCOMP;
+    } else if(flags & VL_FLISBZIP){
+      cmode = VL_OXCOMP;
+    }
   } else if(omode & VL_OWRITER){
-    zmode = omode & VL_OZCOMP;
+    if(omode & VL_OZCOMP){
+      cmode = VL_OZCOMP;
+    } else if(omode & VL_OYCOMP){
+      cmode = VL_OYCOMP;
+    } else if(omode & VL_OXCOMP){
+      cmode = VL_OXCOMP;
+    }
   }
   if(omode & VL_OWRITER){
     flags |= VL_FLISVILLA;
-    if(_qdbm_deflate && zmode) flags |= VL_FLISZLIB;
-    if(!dpsetflags(depot, flags)){
+    if(_qdbm_deflate && cmode == VL_OZCOMP){
+      flags |= VL_FLISZLIB;
+    } else if(_qdbm_lzoencode && cmode == VL_OYCOMP){
+      flags |= VL_FLISLZO;
+    } else if(_qdbm_lzoencode && cmode == VL_OXCOMP){
+      flags |= VL_FLISBZIP;
+    }
+    if(!dpsetflags(depot, flags) || !dpsetfbpsiz(depot, VL_FBPOOLSIZ)){
       dpclose(depot);
       return NULL;
     }
@@ -134,7 +158,7 @@ VILLA *vlopen(const char *name, int omode, VLCFUNC cmp){
   villa->depot = depot;
   villa->cmp = cmp;
   villa->wmode = (omode & VL_OWRITER);
-  villa->zmode = zmode;
+  villa->cmode = cmode;
   villa->root = root;
   villa->last = last;
   villa->lnum = lnum;
@@ -747,6 +771,126 @@ char *vlcurval(VILLA *villa, int *sp){
 }
 
 
+/* Insert a record around the cursor. */
+int vlcurput(VILLA *villa, const char *vbuf, int vsiz, int cpmode){
+  VLLEAF *leaf;
+  VLREC *recp;
+  char *tbuf;
+  int tsiz;
+  assert(villa && vbuf);
+  if(!villa->wmode){
+    dpecodeset(DP_EMODE, __FILE__, __LINE__);
+    return FALSE;
+  }
+  if(vsiz < 0) vsiz = strlen(vbuf);
+  if(villa->curleaf == -1){
+    dpecodeset(DP_ENOITEM, __FILE__, __LINE__);
+    return FALSE;
+  }
+  if(!(leaf = vlleafload(villa, villa->curleaf))){
+    villa->curleaf = -1;
+    return FALSE;
+  }
+  recp = (VLREC *)CB_LISTVAL(leaf->recs, villa->curknum, NULL);
+  switch(cpmode){
+  case VL_CPBEFORE:
+    if(villa->curvnum < 1){
+      if(!recp->rest){
+        tbuf = cbdatumtomalloc(recp->first, &tsiz);
+        recp->first = cbdatumopen(vbuf, vsiz);
+        recp->rest = cblistopen();
+        cblistpushbuf(recp->rest, tbuf, tsiz);
+      } else {
+        cblistunshift(recp->rest, CB_DATUMPTR(recp->first), CB_DATUMSIZE(recp->first));
+        cbdatumsetsize(recp->first, 0);
+        cbdatumcat(recp->first, vbuf, vsiz);
+      }
+    } else {
+      cblistinsert(recp->rest, villa->curvnum - 1, vbuf, vsiz);
+    }
+    villa->rnum++;
+    break;
+  case VL_CPAFTER:
+    if(!recp->rest) recp->rest = cblistopen();
+    cblistinsert(recp->rest, villa->curvnum, vbuf, vsiz);
+    villa->curvnum++;
+    villa->rnum++;
+    break;
+  default:
+    if(villa->curvnum < 1){
+      cbdatumsetsize(recp->first, 0);
+      cbdatumcat(recp->first, vbuf, vsiz);
+    } else {
+      cblistover(recp->rest, villa->curvnum - 1, vbuf, vsiz);
+    }
+    break;
+  }
+  leaf->dirty = TRUE;
+  return TRUE;
+}
+
+
+/* Delete the record where the cursor is. */
+int vlcurout(VILLA *villa){
+  VLLEAF *leaf;
+  VLREC *recp;
+  char *vbuf;
+  int vsiz;
+  assert(villa);
+  if(!villa->wmode){
+    dpecodeset(DP_EMODE, __FILE__, __LINE__);
+    return FALSE;
+  }
+  if(villa->curleaf == -1){
+    dpecodeset(DP_ENOITEM, __FILE__, __LINE__);
+    return FALSE;
+  }
+  if(!(leaf = vlleafload(villa, villa->curleaf))){
+    villa->curleaf = -1;
+    return FALSE;
+  }
+  recp = (VLREC *)CB_LISTVAL(leaf->recs, villa->curknum, NULL);
+  if(villa->curvnum < 1){
+    if(recp->rest){
+      vbuf = cblistshift(recp->rest, &vsiz);
+      cbdatumsetsize(recp->first, 0);
+      cbdatumcat(recp->first, vbuf, vsiz);
+      free(vbuf);
+      if(CB_LISTNUM(recp->rest) < 1){
+        cblistclose(recp->rest);
+        recp->rest = NULL;
+      }
+    } else {
+      cbdatumclose(recp->first);
+      cbdatumclose(recp->key);
+      free(cblistremove(leaf->recs, villa->curknum, NULL));
+    }
+  } else {
+    free(cblistremove(recp->rest, villa->curvnum - 1, NULL));
+    if(villa->curvnum - 1 >= CB_LISTNUM(recp->rest)){
+      villa->curknum++;
+      villa->curvnum = 0;
+    }
+    if(CB_LISTNUM(recp->rest) < 1){
+      cblistclose(recp->rest);
+      recp->rest = NULL;
+    }
+  }
+  villa->rnum--;
+  leaf->dirty = TRUE;
+  if(villa->curknum >= CB_LISTNUM(leaf->recs)){
+    villa->curleaf = leaf->next;
+    villa->curknum = 0;
+    villa->curvnum = 0;
+    while(villa->curleaf != -1 && (leaf = vlleafload(villa, villa->curleaf)) != NULL &&
+          CB_LISTNUM(leaf->recs) < 1){
+      villa->curleaf = leaf->next;
+    }
+  }
+  return TRUE;
+}
+
+
 /* Set the tuning parameters for performance. */
 void vlsettuning(VILLA *villa, int lrecmax, int nidxmax, int lcnum, int ncnum){
   assert(villa);
@@ -876,7 +1020,7 @@ int vlinode(VILLA *villa){
 
 
 /* Get the last modified time of a database. */
-int vlmtime(VILLA *villa){
+time_t vlmtime(VILLA *villa){
   assert(villa);
   return dpmtime(villa->depot);
 }
@@ -1532,17 +1676,45 @@ static int vlleafsave(VILLA *villa, VLLEAF *leaf){
       }
     }
   }
-  if(_qdbm_deflate && villa->zmode){
+  if(_qdbm_deflate && villa->cmode == VL_OZCOMP){
     if(!(zbuf = _qdbm_deflate(CB_DATUMPTR(buf), CB_DATUMSIZE(buf), &zsiz, _QDBM_ZMRAW))){
       cbdatumclose(buf);
-      if(dpecode == DP_EMODE) dpecodeset(DP_EMISC, __FILE__, __LINE__);
+      dpecodeset(DP_EMISC, __FILE__, __LINE__);
       return FALSE;
     }
     villa->avglsiz = (villa->avglsiz * 9 + zsiz) / 10;
     if(!dpsetalign(villa->depot, (int)(villa->avglsiz * VL_ALIGNRATIO)) ||
        !dpput(villa->depot, (char *)&(leaf->id), sizeof(int), zbuf, zsiz, DP_DOVER)){
       cbdatumclose(buf);
-      if(dpecode == DP_EMODE) dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
+      dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
+      return FALSE;
+    }
+    free(zbuf);
+  } else if(_qdbm_lzoencode && villa->cmode == VL_OYCOMP){
+    if(!(zbuf = _qdbm_lzoencode(CB_DATUMPTR(buf), CB_DATUMSIZE(buf), &zsiz))){
+      cbdatumclose(buf);
+      dpecodeset(DP_EMISC, __FILE__, __LINE__);
+      return FALSE;
+    }
+    villa->avglsiz = (villa->avglsiz * 9 + zsiz) / 10;
+    if(!dpsetalign(villa->depot, (int)(villa->avglsiz * VL_ALIGNRATIO)) ||
+       !dpput(villa->depot, (char *)&(leaf->id), sizeof(int), zbuf, zsiz, DP_DOVER)){
+      cbdatumclose(buf);
+      dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
+      return FALSE;
+    }
+    free(zbuf);
+  } else if(_qdbm_bzencode && villa->cmode == VL_OXCOMP){
+    if(!(zbuf = _qdbm_bzencode(CB_DATUMPTR(buf), CB_DATUMSIZE(buf), &zsiz))){
+      cbdatumclose(buf);
+      dpecodeset(DP_EMISC, __FILE__, __LINE__);
+      return FALSE;
+    }
+    villa->avglsiz = (villa->avglsiz * 9 + zsiz) / 10;
+    if(!dpsetalign(villa->depot, (int)(villa->avglsiz * VL_ALIGNRATIO)) ||
+       !dpput(villa->depot, (char *)&(leaf->id), sizeof(int), zbuf, zsiz, DP_DOVER)){
+      cbdatumclose(buf);
+      dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
       return FALSE;
     }
     free(zbuf);
@@ -1552,7 +1724,7 @@ static int vlleafsave(VILLA *villa, VLLEAF *leaf){
        !dpput(villa->depot, (char *)&(leaf->id), sizeof(int),
               CB_DATUMPTR(buf), CB_DATUMSIZE(buf), DP_DOVER)){
       cbdatumclose(buf);
-      if(dpecode == DP_EMODE) dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
+      dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
       return FALSE;
     }
   }
@@ -1567,7 +1739,7 @@ static int vlleafsave(VILLA *villa, VLLEAF *leaf){
    `id' specifies the ID number of the leaf.
    If successful, the return value is the pointer to the leaf, else, it is `NULL'. */
 static VLLEAF *vlleafload(VILLA *villa, int id){
-  char *buf, *rp, *kbuf, *vbuf, *zbuf;
+  char wbuf[VL_PAGEBUFSIZ], *buf, *rp, *kbuf, *vbuf, *zbuf;
   int i, size, step, ksiz, vnum, vsiz, prev, next, zsiz;
   VLLEAF *leaf, lent;
   VLREC rec;
@@ -1579,12 +1751,33 @@ static VLLEAF *vlleafload(VILLA *villa, int id){
   ksiz = -1;
   prev = -1;
   next = -1;
-  if(!(buf = dpget(villa->depot, (char *)&id, sizeof(int), 0, -1, &size))){
+  if((size = dpgetwb(villa->depot, (char *)&id, sizeof(int), 0, VL_PAGEBUFSIZ, wbuf)) > 0 &&
+     size < VL_PAGEBUFSIZ){
+    buf = NULL;
+  } else if(!(buf = dpget(villa->depot, (char *)&id, sizeof(int), 0, -1, &size))){
     dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
     return NULL;
   }
-  if(_qdbm_inflate && villa->zmode){
-    if(!(zbuf = _qdbm_inflate(buf, size, &zsiz, _QDBM_ZMRAW))){
+  if(_qdbm_inflate && villa->cmode == VL_OZCOMP){
+    if(!(zbuf = _qdbm_inflate(buf ? buf : wbuf, size, &zsiz, _QDBM_ZMRAW))){
+      dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
+      free(buf);
+      return NULL;
+    }
+    free(buf);
+    buf = zbuf;
+    size = zsiz;
+  } else if(_qdbm_lzodecode && villa->cmode == VL_OYCOMP){
+    if(!(zbuf = _qdbm_lzodecode(buf ? buf : wbuf, size, &zsiz))){
+      dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
+      free(buf);
+      return NULL;
+    }
+    free(buf);
+    buf = zbuf;
+    size = zsiz;
+  } else if(_qdbm_bzdecode && villa->cmode == VL_OXCOMP){
+    if(!(zbuf = _qdbm_bzdecode(buf ? buf : wbuf, size, &zsiz))){
       dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
       free(buf);
       return NULL;
@@ -1593,7 +1786,7 @@ static VLLEAF *vlleafload(VILLA *villa, int id){
     buf = zbuf;
     size = zsiz;
   }
-  rp = buf;
+  rp = buf ? buf : wbuf;
   if(size >= 1){
     prev = vlreadvnumbuf(rp, size, &step);
     rp += step;
@@ -1660,7 +1853,8 @@ static VLLEAF *vlleafload(VILLA *villa, int id){
 static int vlleafaddrec(VILLA *villa, VLLEAF *leaf, int dmode,
                         const char *kbuf, int ksiz, const char *vbuf, int vsiz){
   VLREC *recp, rec;
-  int i, rv, left, right, ln;
+  int i, rv, left, right, ln, tsiz;
+  char *tbuf;
   assert(villa && leaf && kbuf && ksiz >= 0 && vbuf && vsiz >= 0);
   left = 0;
   ln = CB_LISTNUM(leaf->recs);
@@ -1683,20 +1877,33 @@ static int vlleafaddrec(VILLA *villa, VLLEAF *leaf, int dmode,
     rv = villa->cmp(kbuf, ksiz, CB_DATUMPTR(recp->key), CB_DATUMSIZE(recp->key));
     if(rv == 0){
       switch(dmode){
-      case VL_DOVER:
-        cbdatumsetsize(recp->first, 0);
-        cbdatumcat(recp->first, vbuf, vsiz);
-        break;
       case VL_DKEEP:
         return FALSE;
       case VL_DCAT:
         cbdatumcat(recp->first, vbuf, vsiz);
         break;
-      default:
+      case VL_DDUP:
         if(!recp->rest) recp->rest = cblistopen();
         cblistpush(recp->rest, vbuf, vsiz);
         villa->rnum++;
         break;
+      case VL_DDUPR:
+        if(!recp->rest){
+          tbuf = cbdatumtomalloc(recp->first, &tsiz);
+          recp->first = cbdatumopen(vbuf, vsiz);
+          recp->rest = cblistopen();
+          cblistpushbuf(recp->rest, tbuf, tsiz);
+        } else {
+          cblistunshift(recp->rest, CB_DATUMPTR(recp->first), CB_DATUMSIZE(recp->first));
+          cbdatumsetsize(recp->first, 0);
+          cbdatumcat(recp->first, vbuf, vsiz);
+        }
+        villa->rnum++;
+        break;
+      default:
+        cbdatumsetsize(recp->first, 0);
+        cbdatumcat(recp->first, vbuf, vsiz);
+        break;
       }
       break;
     } else if(rv < 0){
@@ -1825,7 +2032,7 @@ static int vlnodesave(VILLA *villa, VLNODE *node){
      !dpput(villa->depot, (char *)&(node->id), sizeof(int),
             CB_DATUMPTR(buf), CB_DATUMSIZE(buf), DP_DOVER)){
     cbdatumclose(buf);
-    if(dpecode == DP_EMODE) dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
+    dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
     return FALSE;
   }
   cbdatumclose(buf);
@@ -1839,7 +2046,7 @@ static int vlnodesave(VILLA *villa, VLNODE *node){
    `id' specifies the ID number of the node.
    If successful, the return value is the pointer to the node, else, it is `NULL'. */
 static VLNODE *vlnodeload(VILLA *villa, int id){
-  char *buf, *rp, *kbuf;
+  char wbuf[VL_PAGEBUFSIZ], *buf, *rp, *kbuf;
   int size, step, heir, pid, ksiz;
   VLNODE *node, nent;
   VLIDX idx;
@@ -1849,8 +2056,14 @@ static VLNODE *vlnodeload(VILLA *villa, int id){
     return node;
   }
   heir = -1;
-  if(!(buf = dpget(villa->depot, (char *)&id, sizeof(int), 0, -1, &size))) return NULL;
-  rp = buf;
+  if((size = dpgetwb(villa->depot, (char *)&id, sizeof(int), 0, VL_PAGEBUFSIZ, wbuf)) > 0 &&
+     size < VL_PAGEBUFSIZ){
+    buf = NULL;
+  } else if(!(buf = dpget(villa->depot, (char *)&id, sizeof(int), 0, -1, &size))){
+    dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
+    return NULL;
+  }
+  rp = buf ? buf : wbuf;
   if(size >= 1){
     heir = vlreadvnumbuf(rp, size, &step);
     rp += step;
@@ -2053,5 +2266,37 @@ static VLREC *vlrecsearch(VILLA *villa, VLLEAF *leaf, const char *kbuf, int ksiz
 }
 
 
+/* Set the size of the free block pool of a database handle. */
+int vlsetfbpsiz(VILLA *villa, int size){
+  assert(villa && size >= 0);
+  if(!villa->wmode){
+    dpecodeset(DP_EMODE, __FILE__, __LINE__);
+    return FALSE;
+  }
+  return dpsetfbpsiz(villa->depot, size);
+}
+
+
+/* Get flags of a database. */
+int vlgetflags(VILLA *villa){
+  assert(villa);
+  return dpgetflags(villa->depot);
+}
+
+
+/* Set flags of a database.
+   `villa' specifies a database handle connected as a writer.
+   `flags' specifies flags to set.  Lesser ten bits are reserved for internal use.
+   If successful, the return value is true, else, it is false. */
+int vlsetflags(VILLA *villa, int flags){
+  assert(villa);
+  if(!villa->wmode){
+    dpecodeset(DP_EMODE, __FILE__, __LINE__);
+    return FALSE;
+  }
+  return dpsetflags(villa->depot, flags);
+}
+
+
 
 /* END OF FILE */
