@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * The extended advanced API of QDBM
- *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2006 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -56,6 +56,7 @@ extern "C" {
 
 #define VL_DOVER       VST_DOVER
 #define VL_DKEEP       VST_DKEEP
+#define VL_DCAT        VST_DCAT
 #define VL_DDUP        VST_DDUP
 
 #define VL_JFORWARD    VST_JFORWARD
@@ -66,6 +67,7 @@ extern "C" {
 #define vlput          vstput
 #define vlout          vstout
 #define vlget          vstget
+#define vlvsiz         vstvsiz
 #define vlvnum         vstvnum
 #define vlputlist      vstputlist
 #define vloutlist      vstoutlist
@@ -78,6 +80,8 @@ extern "C" {
 #define vlcurjump      vstcurjump
 #define vlcurkey       vstcurkey
 #define vlcurval       vstcurval
+#define vlcurput       vstcurput
+#define vlcurout       vstcurout
 #define vlsettuning    vstsettuning
 #define vlsync         vstsync
 #define vloptimize     vstoptimize
@@ -97,8 +101,14 @@ extern "C" {
 #define vlrepair       vstrepair
 #define vlexportdb     vstexportdb
 #define vlimportdb     vstimportdb
+#define vlgetcache     vstgetcache
+#define vlcurkeycache  vstcurkeycache
+#define vlcurvalcache  vstcurvalcache
+#define vlsetfbpsiz    vstsetfbpsiz
+#define vlgetflags     vstgetflags
+#define vlsetflags     vstsetflags
 
-#ifndef _VISTA_C
+#if !defined(_VISTA_C)
 #include <villa.h>
 #endif
 
@@ -109,11 +119,20 @@ extern "C" {
  *************************************************************************************************/
 
 
+#if defined(_MSC_VER) && !defined(QDBM_INTERNAL) && !defined(QDBM_STATIC)
+#define MYEXTERN extern __declspec(dllimport)
+#else
+#define MYEXTERN extern
+#endif
+
+
 /* Number of division of the database. */
-extern int vscrdnum;
+MYEXTERN int vscrdnum;
 
 
 
+#undef MYEXTERN
+
 #if defined(__cplusplus)                 /* export for C++ */
 }
 #endif
