@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Test cases of Villa
- *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2006 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -53,13 +53,13 @@ int runwicked(int argc, char **argv);
 int printfflush(const char *format, ...);
 void pdperror(const char *name);
 int myrand(void);
-int dowrite(const char *name, int rnum, int ii, int zc,
+int dowrite(const char *name, int rnum, int ii, int cmode,
             int lrecmax, int nidxmax, int lcnum, int ncnum);
 int doread(const char *name, int ii, int vc);
-int dordup(const char *name, int rnum, int pnum, int ii, int zc, int cc,
+int dordup(const char *name, int rnum, int pnum, int ii, int cmode, int cc,
            int lrecmax, int nidxmax, int lcnum, int ncnum);
-int docombo(const char *name, int zc);
-int dowicked(const char *name, int rnum, int cb, int zc);
+int docombo(const char *name, int cmode);
+int dowicked(const char *name, int rnum, int cb, int cmode);
 
 
 /* main routine */
@@ -93,13 +93,13 @@ void usage(void){
   fprintf(stderr, "%s: test cases for Villa\n", progname);
   fprintf(stderr, "\n");
   fprintf(stderr, "usage:\n");
-  fprintf(stderr, "  %s write [-int] [-zc] [-tune lrecmax nidxmax lcnum ncnum] name rnum\n",
-          progname);
+  fprintf(stderr, "  %s write [-int] [-cz|-cy|-cx] [-tune lrecmax nidxmax lcnum ncnum]"
+          " name rnum\n", progname);
   fprintf(stderr, "  %s read [-int] [-vc] name\n", progname);
-  fprintf(stderr, "  %s rdup [-int] [-zc] [-cc] [-tune lrecmax nidxmax lcnum ncnum]"
+  fprintf(stderr, "  %s rdup [-int] [-cz|-cy|-cx] [-cc] [-tune lrecmax nidxmax lcnum ncnum]"
           " name rnum pnum\n", progname);
-  fprintf(stderr, "  %s combo [-zc] name\n", progname);
-  fprintf(stderr, "  %s wicked [-c] [-zc] name rnum\n", progname);
+  fprintf(stderr, "  %s combo [-cz|-cy|-cx] name\n", progname);
+  fprintf(stderr, "  %s wicked [-c] [-cz|-cy|-cx] name rnum\n", progname);
   fprintf(stderr, "\n");
   exit(1);
 }
@@ -108,12 +108,12 @@ void usage(void){
 /* parse arguments of write command */
 int runwrite(int argc, char **argv){
   char *name, *rstr;
-  int i, rnum, ii, zc, lrecmax, nidxmax, lcnum, ncnum, rv;
+  int i, rnum, ii, cmode, lrecmax, nidxmax, lcnum, ncnum, rv;
   name = NULL;
   rstr = NULL;
   rnum = 0;
   ii = FALSE;
-  zc = FALSE;
+  cmode = 0;
   lrecmax = -1;
   nidxmax = -1;
   lcnum = -1;
@@ -122,8 +122,12 @@ int runwrite(int argc, char **argv){
     if(!name && argv[i][0] == '-'){
       if(!strcmp(argv[i], "-int")){
         ii = TRUE;
-      } else if(!strcmp(argv[i], "-zc")){
-        zc = TRUE;
+      } else if(!strcmp(argv[i], "-cz")){
+        cmode |= VL_OZCOMP;
+      } else if(!strcmp(argv[i], "-cy")){
+        cmode |= VL_OYCOMP;
+      } else if(!strcmp(argv[i], "-cx")){
+        cmode |= VL_OXCOMP;
       } else if(!strcmp(argv[i], "-tune")){
         if(++i >= argc) usage();
         lrecmax = atoi(argv[i]);
@@ -147,7 +151,7 @@ int runwrite(int argc, char **argv){
   if(!name || !rstr) usage();
   rnum = atoi(rstr);
   if(rnum < 1) usage();
-  rv = dowrite(name, rnum, ii, zc, lrecmax, nidxmax, lcnum, ncnum);
+  rv = dowrite(name, rnum, ii, cmode, lrecmax, nidxmax, lcnum, ncnum);
   return rv;
 }
 
@@ -183,14 +187,14 @@ int runread(int argc, char **argv){
 /* parse arguments of rdup command */
 int runrdup(int argc, char **argv){
   char *name, *rstr, *pstr;
-  int i, rnum, pnum, ii, zc, cc, lrecmax, nidxmax, lcnum, ncnum, rv;
+  int i, rnum, pnum, ii, cmode, cc, lrecmax, nidxmax, lcnum, ncnum, rv;
   name = NULL;
   rstr = NULL;
   pstr = NULL;
   rnum = 0;
   pnum = 0;
   ii = FALSE;
-  zc = FALSE;
+  cmode = 0;
   cc = FALSE;
   lrecmax = -1;
   nidxmax = -1;
@@ -200,8 +204,12 @@ int runrdup(int argc, char **argv){
     if(!name && argv[i][0] == '-'){
       if(!strcmp(argv[i], "-int")){
         ii = TRUE;
-      } else if(!strcmp(argv[i], "-zc")){
-        zc = TRUE;
+      } else if(!strcmp(argv[i], "-cz")){
+        cmode |= VL_OZCOMP;
+      } else if(!strcmp(argv[i], "-cy")){
+        cmode |= VL_OYCOMP;
+      } else if(!strcmp(argv[i], "-cx")){
+        cmode |= VL_OXCOMP;
       } else if(!strcmp(argv[i], "-cc")){
         cc = TRUE;
       } else if(!strcmp(argv[i], "-tune")){
@@ -230,7 +238,7 @@ int runrdup(int argc, char **argv){
   rnum = atoi(rstr);
   pnum = atoi(pstr);
   if(rnum < 1 || pnum < 1) usage();
-  rv = dordup(name, rnum, pnum, ii, zc, cc, lrecmax, nidxmax, lcnum, ncnum);
+  rv = dordup(name, rnum, pnum, ii, cmode, cc, lrecmax, nidxmax, lcnum, ncnum);
   return rv;
 }
 
@@ -238,13 +246,17 @@ int runrdup(int argc, char **argv){
 /* parse arguments of combo command */
 int runcombo(int argc, char **argv){
   char *name;
-  int i, zc, rv;
+  int i, cmode, rv;
   name = NULL;
-  zc = FALSE;
+  cmode = 0;
   for(i = 2; i < argc; i++){
     if(!name && argv[i][0] == '-'){
-      if(!strcmp(argv[i], "-zc")){
-        zc = TRUE;
+      if(!strcmp(argv[i], "-cz")){
+        cmode |= VL_OZCOMP;
+      } else if(!strcmp(argv[i], "-cy")){
+        cmode |= VL_OYCOMP;
+      } else if(!strcmp(argv[i], "-cx")){
+        cmode |= VL_OXCOMP;
       } else {
         usage();
       }
@@ -255,7 +267,7 @@ int runcombo(int argc, char **argv){
     }
   }
   if(!name) usage();
-  rv = docombo(name, zc);
+  rv = docombo(name, cmode);
   return rv;
 }
 
@@ -263,17 +275,21 @@ int runcombo(int argc, char **argv){
 /* parse arguments of wicked command */
 int runwicked(int argc, char **argv){
   char *name, *rstr;
-  int i, cb, zc, rnum, rv;
+  int i, cb, cmode, rnum, rv;
   name = NULL;
   rstr = NULL;
   cb = FALSE;
-  zc = FALSE;
+  cmode = 0;
   for(i = 2; i < argc; i++){
     if(!name && argv[i][0] == '-'){
       if(!strcmp(argv[i], "-c")){
         cb = TRUE;
-      } else if(!strcmp(argv[i], "-zc")){
-        zc = TRUE;
+      } else if(!strcmp(argv[i], "-cz")){
+        cmode |= VL_OZCOMP;
+      } else if(!strcmp(argv[i], "-cy")){
+        cmode |= VL_OYCOMP;
+      } else if(!strcmp(argv[i], "-cx")){
+        cmode |= VL_OXCOMP;
       } else {
         usage();
       }
@@ -288,7 +304,7 @@ int runwicked(int argc, char **argv){
   if(!name || !rstr) usage();
   rnum = atoi(rstr);
   if(rnum < 1) usage();
-  rv = dowicked(name, rnum, cb, zc);
+  rv = dowicked(name, rnum, cb, cmode);
   return rv;
 }
 
@@ -320,17 +336,16 @@ int myrand(void){
 
 
 /* perform write command */
-int dowrite(const char *name, int rnum, int ii, int zc,
+int dowrite(const char *name, int rnum, int ii, int cmode,
             int lrecmax, int nidxmax, int lcnum, int ncnum){
   VILLA *villa;
   int i, omode, err, len;
   char buf[RECBUFSIZ];
-  printfflush("<Writing Test>\n  name=%s  rnum=%d  int=%d  zc=%d  "
+  printfflush("<Writing Test>\n  name=%s  rnum=%d  int=%d  cmode=%d  "
               "lrecmax=%d  nidxmax=%d  lcnum=%d  ncnum=%d\n\n",
-              name, rnum, ii, zc, lrecmax, nidxmax, lcnum, ncnum);
+              name, rnum, ii, cmode, lrecmax, nidxmax, lcnum, ncnum);
   /* open a database */
-  omode = VL_OWRITER | VL_OCREAT | VL_OTRUNC;
-  if(zc) omode |= VL_OZCOMP;
+  omode = VL_OWRITER | VL_OCREAT | VL_OTRUNC | cmode;
   if(!(villa = vlopen(name, omode, ii ? VL_CMPINT : VL_CMPLEX))){
     pdperror(name);
     return 1;
@@ -444,16 +459,15 @@ int doread(const char *name, int ii, int vc){
 
 
 /* perform rdup command */
-int dordup(const char *name, int rnum, int pnum, int ii, int zc, int cc,
+int dordup(const char *name, int rnum, int pnum, int ii, int cmode, int cc,
            int lrecmax, int nidxmax, int lcnum, int ncnum){
   VILLA *villa;
   int i, omode, err, dmode, vi, len;
   char buf[RECBUFSIZ];
-  printfflush("<Random Writing Test>\n  name=%s  rnum=%d  int=%d  zc=%d  "
+  printfflush("<Random Writing Test>\n  name=%s  rnum=%d  int=%d  cmode=%d  "
               "lrecmax=%d  nidxmax=%d  lcnum=%d  ncnum=%d\n\n",
-              name, rnum, ii, zc, lrecmax, nidxmax, lcnum, ncnum);
-  omode = VL_OWRITER | VL_OCREAT | VL_OTRUNC;
-  if(zc) omode |= VL_OZCOMP;
+              name, rnum, ii, cmode, lrecmax, nidxmax, lcnum, ncnum);
+  omode = VL_OWRITER | VL_OCREAT | VL_OTRUNC | cmode;
   if(!(villa = vlopen(name, omode, ii ? VL_CMPINT : VL_CMPLEX))){
     pdperror(name);
     return 1;
@@ -461,7 +475,7 @@ int dordup(const char *name, int rnum, int pnum, int ii, int zc, int cc,
   err = FALSE;
   if(lrecmax > 0) vlsettuning(villa, lrecmax, nidxmax, lcnum, ncnum);
   for(i = 1; i <= rnum; i++){
-    dmode = VL_DDUP;
+    dmode = i % 3 == 0 ? VL_DDUPR : VL_DDUP;
     if(cc && myrand() % 2 == 0) dmode = VL_DCAT;
     vi = myrand() % pnum + 1;
     if(ii){
@@ -497,16 +511,15 @@ int dordup(const char *name, int rnum, int pnum, int ii, int zc, int cc,
 
 
 /* perform combo command */
-int docombo(const char *name, int zc){
+int docombo(const char *name, int cmode){
   VILLA *villa;
   char buf[RECBUFSIZ], *vbuf, *kbuf;
-  int i, omode, len, vsiz, ksiz, fsiz, lnum, nnum, rnum;
+  int i, j, omode, len, vsiz, ksiz, fsiz, lnum, nnum, rnum;
   CBLIST *alist, *dlist;
   const char *ap, *dp;
-  printfflush("<Combination Test>\n  name=%s  zc=%d\n\n", name, zc);
+  printfflush("<Combination Test>\n  name=%s  cmode=%d\n\n", name, cmode);
   printfflush("Creating a database with VL_CMPLEX ... ");
-  omode = VL_OWRITER | VL_OCREAT | VL_OTRUNC;
-  if(zc) omode |= VL_OZCOMP;
+  omode = VL_OWRITER | VL_OCREAT | VL_OTRUNC | cmode;
   if(!(villa = vlopen(name, omode, VL_CMPLEX))){
     pdperror(name);
     return 1;
@@ -716,10 +729,10 @@ int docombo(const char *name, int zc){
     return 1;
   }
   printfflush("ok\n");
-  printfflush("Adding 50 random records with VL_DDUP ... ");
+  printfflush("Adding 50 random records with VL_DDUPR ... ");
   for(i = 0; i < 50; i++){
     len = sprintf(buf, "%08d", myrand() % 100 + 1);
-    if(!vlput(villa, buf, len, buf, len, VL_DDUP)){
+    if(!vlput(villa, buf, len, buf, len, VL_DDUPR)){
       pdperror(name);
       vlclose(villa);
       return 1;
@@ -997,13 +1010,151 @@ int docombo(const char *name, int zc){
     return 1;
   }
   printfflush("ok\n");
+  printfflush("Creating a database with VL_CMPLEX ... ");
+  omode = VL_OWRITER | VL_OCREAT | VL_OTRUNC | cmode;
+  if(!(villa = vlopen(name, omode, VL_CMPLEX))){
+    pdperror(name);
+    return 1;
+  }
+  printfflush("ok\n");
+  printfflush("Setting tuning parameters with 5, 6, 16, 16 ... ");
+  vlsettuning(villa, 5, 6, 16, 16);
+  printfflush("ok\n");
+  printfflush("Adding 3 * 3 records with VL_DDUP ... ");
+  for(i = 0; i < 3; i++){
+    for(j = 0; j < 3; j++){
+      len = sprintf(buf, "%08d", j);
+      if(!vlput(villa, buf, len, buf, -1, VL_DDUP)){
+        pdperror(name);
+        vlclose(villa);
+        return 1;
+      }
+    }
+  }
+  printfflush("ok\n");
+  printfflush("Inserting records with the cursor ... ");
+  if(!vlcurjump(villa, "00000001", -1, VL_JFORWARD) ||
+     !vlcurput(villa, "first", -1, VL_CPAFTER) || !vlcurput(villa, "second", -1, VL_CPAFTER) ||
+     !vlcurnext(villa) ||
+     !vlcurput(villa, "third", -1, VL_CPAFTER) ||
+     strcmp(vlcurvalcache(villa, NULL), "third") ||
+     !vlcurput(villa, "fourth", -1, VL_CPCURRENT) ||
+     strcmp(vlcurvalcache(villa, NULL), "fourth") ||
+     !vlcurjump(villa, "00000001", -1, VL_JFORWARD) ||
+     strcmp(vlcurvalcache(villa, NULL), "00000001") ||
+     !vlcurput(villa, "one", -1, VL_CPBEFORE) || !vlcurput(villa, "two", -1, VL_CPBEFORE) ||
+     !vlcurput(villa, "three", -1, VL_CPBEFORE) || !vlcurput(villa, "five", -1, VL_CPBEFORE) ||
+     !vlcurnext(villa) ||
+     !vlcurput(villa, "four", -1, VL_CPBEFORE) ||
+     strcmp(vlcurvalcache(villa, NULL), "four") ||
+     !vlcurjump(villa, "00000001*", -1, VL_JBACKWARD) ||
+     strcmp(vlcurvalcache(villa, NULL), "00000001") ||
+     !vlcurput(villa, "omega", -1, VL_CPAFTER) ||
+     strcmp(vlcurkeycache(villa, NULL), "00000001") ||
+     strcmp(vlcurvalcache(villa, NULL), "omega") ||
+     !vlcurjump(villa, "00000000*", -1, VL_JFORWARD) ||
+     !vlcurput(villa, "alpha", -1, VL_CPBEFORE) ||
+     strcmp(vlcurvalcache(villa, NULL), "alpha") ||
+     !vlcurprev(villa) ||
+     strcmp(vlcurkeycache(villa, NULL), "00000000") ||
+     strcmp(vlcurvalcache(villa, NULL), "00000000") ||
+     !vlcurput(villa, "before", -1, VL_CPAFTER) ||
+     strcmp(vlcurvalcache(villa, NULL), "before") ||
+     !vlcurjump(villa, "00000001*", -1, VL_JFORWARD) ||
+     !vlcurput(villa, "after", -1, VL_CPBEFORE) ||
+     strcmp(vlcurvalcache(villa, NULL), "after") ||
+     !vlcurfirst(villa) ||
+     strcmp(vlcurvalcache(villa, NULL), "00000000") ||
+     !vlcurput(villa, "top", -1, VL_CPBEFORE) ||
+     strcmp(vlcurvalcache(villa, NULL), "top") ||
+     !vlcurlast(villa) ||
+     !vlcurput(villa, "bottom", -1, VL_CPAFTER) ||
+     strcmp(vlcurvalcache(villa, NULL), "bottom")){
+    fprintf(stderr, "%s: %s: invalid\n", progname, name);
+    vlclose(villa);
+    return 1;
+  }
+  printfflush("ok\n");
+
+  printfflush("Deleting records with the cursor ... ");
+  if(!vlcurjump(villa, "00000000*", -1, VL_JBACKWARD) ||
+     strcmp(vlcurvalcache(villa, NULL), "before") ||
+     !vlcurout(villa) ||
+     strcmp(vlcurvalcache(villa, NULL), "alpha") ||
+     !vlcurout(villa) ||
+     strcmp(vlcurvalcache(villa, NULL), "five") ||
+     !vlcurfirst(villa) || !vlcurnext(villa) ||
+     !vlcurout(villa) || !vlcurout(villa) || !vlcurout(villa) ||
+     strcmp(vlcurvalcache(villa, NULL), "five") ||
+     !vlcurprev(villa) ||
+     strcmp(vlcurvalcache(villa, NULL), "top") ||
+     !vlcurout(villa) ||
+     strcmp(vlcurvalcache(villa, NULL), "five") ||
+     !vlcurjump(villa, "00000002", -1, VL_JBACKWARD) ||
+     strcmp(vlcurvalcache(villa, NULL), "bottom") ||
+     !vlcurout(villa) ||
+     !vlcurjump(villa, "00000001", -1, VL_JBACKWARD) ||
+     !vlcurout(villa) ||
+     !vlcurout(villa) || !vlcurout(villa) || !vlcurout(villa) ||
+     strcmp(vlcurkeycache(villa, NULL), "00000002") ||
+     strcmp(vlcurvalcache(villa, NULL), "00000002") ||
+     !vlcurout(villa) || vlcurout(villa) ||
+     !vlcurfirst(villa) ||
+     strcmp(vlcurvalcache(villa, NULL), "five")){
+    fprintf(stderr, "%s: %s: invalid\n", progname, name);
+    vlclose(villa);
+    return 1;
+  }
+  vlcurfirst(villa);
+  while(vlcurout(villa)){
+    free(vlcurval(villa, NULL));
+  }
+  if(vlrnum(villa) != 0){
+    printf("%d\n", vlrnum(villa));
+    fprintf(stderr, "%s: %s: invalid\n", progname, name);
+    vlclose(villa);
+    return 1;
+  }
+  for(i = 0; i < 1000; i++){
+    len = sprintf(buf, "%08d", i);
+    if(!vlput(villa, buf, len, buf, -1, VL_DKEEP)){
+      pdperror(name);
+      vlclose(villa);
+      return 1;
+    }
+  }
+  for(i = 200; i < 800; i++){
+    len = sprintf(buf, "%08d", i);
+    if(!vlout(villa, buf, len)){
+      pdperror(name);
+      vlclose(villa);
+      return 1;
+    }
+  }
+  vlcurfirst(villa);
+  while(vlcurout(villa)){
+    free(vlcurval(villa, NULL));
+  }
+  if(vlrnum(villa) != 0){
+    printf("%d\n", vlrnum(villa));
+    fprintf(stderr, "%s: %s: invalid\n", progname, name);
+    vlclose(villa);
+    return 1;
+  }
+  printfflush("ok\n");
+  printfflush("Closing the database ... ");
+  if(!vlclose(villa)){
+    pdperror(name);
+    return 1;
+  }
+  printfflush("ok\n");
   printfflush("all ok\n\n");
   return 0;
 }
 
 
 /* perform wicked command */
-int dowicked(const char *name, int rnum, int cb, int zc){
+int dowicked(const char *name, int rnum, int cb, int cmode){
   VILLA *villa;
   CBMAP *map;
   int i, j, omode, len, err, ksiz, vsiz, tran, mksiz, mvsiz, rsiz;
@@ -1011,8 +1162,7 @@ int dowicked(const char *name, int rnum, int cb, int zc){
   char buf[32], *kbuf, *vbuf;
   CBLIST *list;
   printfflush("<Wicked Writing Test>\n  name=%s  rnum=%d\n\n", name, rnum);
-  omode = VL_OWRITER | VL_OCREAT | VL_OTRUNC;
-  if(zc) omode |= VL_OZCOMP;
+  omode = VL_OWRITER | VL_OCREAT | VL_OTRUNC | cmode;
   if(!(villa = vlopen(name, omode, VL_CMPLEX))){
     pdperror(name);
     return 1;
@@ -1122,7 +1272,32 @@ int dowicked(const char *name, int rnum, int cb, int zc){
       }
       break;
     case 11:
-      if(myrand() % 32 != 0){
+      putchar('L');
+      if(myrand() % 3 == 0 &&
+         !vlcurjump(villa, buf, len, i % 3 == 0 ? VL_JFORWARD : VL_JBACKWARD) &&
+         dpecode != DP_ENOITEM) err = TRUE;
+      for(j = myrand() % 5; j >= 0; j--){
+        switch(myrand() % 6){
+        case 0:
+          if(!vlcurput(villa, buf, len, VL_CPAFTER) && dpecode != DP_ENOITEM) err = TRUE;
+          break;
+        case 1:
+          if(!vlcurput(villa, buf, len, VL_CPBEFORE) && dpecode != DP_ENOITEM) err = TRUE;
+          break;
+        case 2:
+          if(!vlcurput(villa, buf, len, VL_CPCURRENT) && dpecode != DP_ENOITEM) err = TRUE;
+          break;
+        default:
+          if(!vlcurout(villa)){
+            if(dpecode != DP_ENOITEM) err = TRUE;
+            break;
+          }
+          break;
+        }
+      }
+      break;
+    case 12:
+      if(tran ? myrand() % 32 != 0 : myrand() % 1024 != 0){
         putchar('N');
         break;
       }
@@ -1141,7 +1316,7 @@ int dowicked(const char *name, int rnum, int cb, int zc){
       break;
     default:
       putchar('P');
-      if(!vlput(villa, buf, len, buf, len, VL_DOVER)) err = TRUE;
+      if(!vlput(villa, buf, len, buf, len, myrand() % 3 == 0 ? VL_DDUPR : VL_DDUP)) err = TRUE;
       break;
     }
     if(i % 50 == 0) printfflush(" (%08d)\n", i);
